;
;*******************************************************
;*******************************************************
;**                                                   **
;**        Monitor szeregowy z moduem LCD            **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
;	Realizowane sekwencje sterujace:
;	<Cr>		- przechodzi na poczatek wlasnego wiersza
;	<Lf>		- przechodzi do nastpnego wiersza
;	<ESC> H		- funkcja przestawia kursor na poczatek ekranu
;	<ESC> J		- funkcja czysci ekran
;	<ESC> C 0	- funkcja wylacza kursor
;	<ESC> C 1	- funkcja wlacza kursor
;	<ESC> Y <m>:<n>	- ustawienie kursora w wierszu <n> kolumnie <m>
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
;			= r0 [reserved]
.def TTYCharParam	= r13
.def TTYServiceL	= r14
.def TTYServiceH	= r15
.def acc		= r16
.def acc2		= r17
.def param		= r18
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : LDCD data bus
;					  ||||||+--- output : LDCD data bus
;					  |||||+---- output : LDCD data bus
;					  ||||+----- output : LDCD data bus
;					  |||+------ output : LDCD data bus
;					  ||+------- output : LDCD data bus
;					  |+-------- output : LDCD data bus
;					  +--------- output : LDCD data bus
.equ		Stand_PAData	=	0b11111111

;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : KEY
;					  ||||||+--- input : KEY
;					  |||||+---- input : KEY
;					  ||||+----- input : KEY
;					  |||+------ input : KEY
;					  ||+------- input : KEY
;					  |+-------- input : KEY
;					  +--------- input : KEY
.equ		Stand_PBData	=	0b11111111

;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : lcd E
;					  ||||||+--- output : lcd RW
;					  |||||+---- output : lcd RS
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11000001
;					  ||||||||
;					  |||||||+-- output: TXD
;					  ||||||+--- input : RXD
;					  |||||+---- input :
;					  ||||+----- input :
;					  |||+------ input :
;					  ||+------- input :
;					  |+-------- output : TXD LED
;					  +--------- output : RXD LED
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
.equ	Key0Pin		= 7
.equ	Key1Pin		= 6
.equ	Key2Pin		= 5
.equ	Key3Pin		= 4
.equ	Key4Pin		= 3
.equ	Key5Pin		= 2
.equ	Key6Pin		= 1
.equ	Key7Pin		= 0
;-----------------------------------------------------------------------------
.equ	Key0Code	= '0'
.equ	Key1Code	= '1'
.equ	Key2Code	= '2'
.equ	Key3Code	= '3'
.equ	Key4Code	= '4'
.equ	Key5Code	= '5'
.equ	Key6Code	= '6'
.equ	Key7Code	= '7'
;-----------------------------------------------------------------------------
.equ UBRRForSpeed2400	= 191
.equ UBRRForSpeed4800	= 95
.equ UBRRForSpeed9600	= 47
.equ UBRRForSpeed19200	= 23
.equ UBRRForSpeed38400	= 11
.equ UBRRForSpeed115200	= 3
;-----------------------------------------------------------------------------
		.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
						;CONST
.equ	DefCharSize	= 8			; DefCharSize = 8 ;
.equ	Cr		= 0x0D			; Cr = 0x0D ;
.equ	Lf		= 0x0A			; Lf = 0x0A ;
.equ	Esc		= 0x1B			; Esc = 0x1B ;
						;
						;VAR
SerialSpeed :		.byte	1		; SerialSpeed : BYTE ;
Idle :			.byte	1		; Idle : BOOLEAN ;
SecondLine :		.byte	1		; SecondLine : BOOLEAN ;
Col :			.byte	1		; Col : BYTE ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;-----------------------------------------------------------------------------
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	rjmp	USARTRXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	rjmp	USARTDataRegEmptInterrupt	;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	rjmp	USARTTXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
.equ	lcd_E_port	= PORTC
.equ	lcd_E_pin	= 0
.equ	lcd_RW_port	= PORTC
.equ	lcd_RW_pin	= 1
.equ	lcd_RS_port	= PORTC
.equ	lcd_RS_pin	= 2
.equ	lcd_Data_port	= PORTA
.include "lcdlib.inc"
;-----------------------------------------------------------------------------
.equ	KeyPort		= PORTB
.equ	KeyPDirection	= KeyPort - 1
.equ	KeyPInput	= KeyPort - 2
.include "keybsrv.inc"
;-----------------------------------------------------------------------------
.equ	USART_LED_Trace	= 1
.equ	RXDLED		= 7
.equ	TXDLED		= 6
.equ	USARTTracePort	= PORTD
.equ	LEDOnState	= 0
.equ	TXDBuffSize	= 32
.equ	RXDBuffSize	= 128
.include "usartlib.inc"
;-----------------------------------------------------------------------------
TC0OvfInterrupt	:				;PROCEDURE TC0OvfInterrupt ( ) ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	rcall	KeybService			; KeybService ( ) ;
.if USART_LED_Trace == 1
	rcall	USARTTimeService		; USARTTimeService ( ) ;
.endif
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	rcall	InitLCDEnvir			; InitLCDEnvir ( ) ;
	ldi	acc,0				; KeyPort := <input> + <pullup>;
	out	KeyPDirection,acc		;
	ldi	acc,0xFF			;
	out	KeyPort,acc			;
	ldz	UBRRForSpeed9600		; USARTHardwareInit ( UBRRForSpeed9600 ) ;
	rcall	USARTHardwareInit		;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	ldi	acc,0				;
	sts	SecondLine,acc			; SecondLine := FALSE ;
	ldi	acc,1				; Idle := TRUE ;
	sts	Idle,acc			;
	rcall	KeybSoftInit			; KeybSoftInit ( ) ;
	ldi	acc,UBRRForSpeed9600		; SerialSpeed := UBRRForSpeed9600 ;
	sts	SerialSpeed,acc			;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
DecodeKey :					;PROCEDURE DecodeKey ( ) ;
;**********					;VAR
;						; Kbd [acc2] : BYTE ;
;						; Temp [acc] : BYTE ;
						;BEGIN (* DecodeKey *)
	in	acc2,KeyPInput			; Kbd := KeyPInput ;
	ldi	acc,0				; Temp := 0 ;
	sbrs	acc2,Key0Pin			; IF Kbd[Key0Pin] THEN
	ldi	acc,Key0Code			;  Temp := Key0Code ;
	sbrs	acc2,Key1Pin			; IF Kbd[Key1Pin] THEN
	ldi	acc,Key1Code			;  Temp := Key1Code ;
	sbrs	acc2,Key2Pin			; IF Kbd[Key2Pin] THEN
	ldi	acc,Key2Code			;  Temp := Key2Code ;
	sbrs	acc2,Key3Pin			; IF Kbd[Key3Pin] THEN
	ldi	acc,Key3Code			;  Temp := Key3Code ;
	sbrs	acc2,Key4Pin			; IF Kbd[Key4Pin] THEN
	ldi	acc,Key4Code			;  Temp := Key4Code ;
	sbrs	acc2,Key5Pin			; IF Kbd[Key5Pin] THEN
	ldi	acc,Key5Code			;  Temp := Key5Code ;
	sbrs	acc2,Key6Pin			; IF Kbd[Key6Pin] THEN
	ldi	acc,Key6Code			;  Temp := Key6Code ;
	sbrs	acc2,Key7Pin			; IF Kbd[Key7Pin] THEN
	ldi	acc,Key7Code			;  Temp := Key7Code ;
	rcall	StoreKey			; StoreKey ( Temp ) ;
	ret					;END (* DecodeKey *) ;
;-----------------------------------------------------------------------------
FindByteInArr :					;PROCEDURE FindByteInArr ( Data [param] : BYTE ;
;****************			   	;		Array [z] : ARRAY OF BYTE ) : BYTE[acc] ;
;						;VAR
;						; Loop[acc2] : BYTE ;
;						; Ch[acc] : CHAR ;
;						;BEGIN (* FindByteInArr *)
	clr	acc2				; Loop := 0 ;
FiByIA_2 :					; REPEAT
	lpm					;  Ch := Array [ Loop ] ;
	mov	acc,r0				;
	cpi	acc,0xFF			;  IF Ch = 0x0FF THEN
	brne	FiByIA_0			;  BEGIN
	ldi	acc,0xFF			;   RETURN 0xFF ;
	ret					;
FiByIA_0 :					;  END (* IF *) ;
	cp	acc,param			;  IF Ch = Data THEN
	brne	FiByIA_1			;  BEGIN
	mov	acc,acc2			;   RETURN Loop ;
	ret					;
FiByIA_1 :					;  END (* IF *) ;
	adiw	zl,1				;
	inc	acc2				;
	rjmp	FiByIA_2			; UNTIL FALSE ;
						;END (* FindByteInArr *) ;
;-----------------------------------------------------------------------------
RunService :					;PROCEDURE RunService ( Inx [param] ; BYTE :
;****************				;			Serv [z] : ADDRESS ) ;
;						;BEGIN (* RunService *)
	lsl	param				; Serv [ Inx ] ( ) ;
	clr	acc				;
	add	zl,param			;
	adc	zh,acc				;
	lpm					;
	push	r0				;
	adiw	zl,1				;
	lpm					;
	mov	zh,r0				;
	pop	zl				;
	icall					;
	ret					;END (* RunService *) ;
;-----------------------------------------------------------------------------
AnyKeyActive :					;FUNCTION AnyKeyActive ( ) : BOOLEAN [C] ;
;*************					;BEGIN (* AnyKeyActive *)
	in	acc,KeyPInput			; IF KeyPort = 0xFF THEN
	cpi	acc,0xFF			;
	brne	AnKeAct_0			; BEGIN
	clc					;  AnyKeyActive := FALSE ;
	ret					;  RETURN ;
AnKeAct_0 :					; END (* IF *) ;
	sec					; AnyKeyActive := TRUE ;
	ret					;END (* AnyKeyActive *) ;
;-----------------------------------------------------------------------------
AnyKey	:					;PROCEDURE AnyKey ( ) ;
;********					;BEGIN (* AnyKey *)
	rcall	NewLineLCD			; NewLineLCD ( ) ;
	ldz	AnyKeyTxt<<1			; WriteTextLCDFlash ( AnyKeyTxt ) ;
	rcall	WriteTextLCDFlash		;
AnyK_0	:					; WHILE NOT KeyPressed ( ) DO
	rcall	KeyPressed			; BEGIN
	brcc	AnyK_0				; END (* WHILE *) ;
	rcall	ReadKey				; ? := ReadKey ( ) ;
	ret					;END (* AnyKey *) ;
;-----------------------------------------------------------------------------
DisplaySerialSpeed :				;PROCEDURE DisplaySerialSpeed ( ) ;
;*******************				;VAR
;						; Inx[acc] : BYTE ;
;						;BEGIN (* DisplaySerialSpeed *)
	lds	param,SerialSpeed		; Inx := FindByteInArr ( SerialSpeed , SpeedCodeTable ) ;
	ldz	SpeedCodeTable<<1		;
	rcall	FindByteInArr			;
	cpi	acc,0xFF			; IF Inx = 0xFF THEN
	brne	DiSeSp_0			; BEGIN
	ldz	UnknownTxt<<1			;  WriteTextLCDFlash ( UnknownTxt ) ;
	rcall	WriteTextLCDFlash		;
	ret					;  RETURN ;
DiSeSp_0 :					; END (* IF *) ;
	ldz	DispSpeedServiceTable<<1	; WriteTextLCDFlash ( DispSpeedServiceTable [ Inx ] ) ;
	lsl	acc				;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;
	push	r0				;
	adiw	zl,1				;
	lpm					;
	mov	zh,r0				;
	pop	zl				;
	rcall	WriteTextLCDFlash		;
	ret					;END (* DisplaySerialSpeed *) ;
;-----------------------------------------------------------------------------
DisplaySerialParam :				;PROCEDURE DisplaySerialParam ( ) ;
;*******************				;BEGIN (* DisplaySerialParam *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	SerialParamPrompt<<1		; WriteTextLCDFlash ( SerialParamPrompt ) ;
	rcall	WriteTextLCDFlash		;
	ldz	SerialMode<<1			; WriteTextLCDFlash ( SerialMode ) ;
	rcall	WriteTextLCDFlash		;
	rcall	DisplaySerialSpeed		; DisplaySerialSpeed ( ) ;
	rcall	AnyKey				; AnyKey ( ) ;
	rcall	ClrScrLCDService		; ClrScrLCDService ( ) ;
	ret					;END (* DisplaySerialParam *) ;
;-----------------------------------------------------------------------------
SetupService :					;PROCEDURE SetupService ( ) ;
;************					;VAR
;						; Inx[acc] : BYTE ;
;						;BEGIN (* SetupService *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	SetupMenuTxt<<1			; WriteTextLCDFlash ( SetupMenuTxt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	NewLineLCD			; NewLineLCD ( ) ;
	ldz	SetupMenuTxt2<<1		; WriteTextLCDFlash ( SetupMenuTxt2 ) ;
	rcall	WriteTextLCDFlash		;
SeSrv_0	:					; WHILE NOT KeyPressed ( ) DO
	rcall	KeyPressed			; BEGIN
	brcc	SeSrv_0				; END (* WHILE *) ;
	rcall	ReadKey				; Inx := FindByteInArr ( ReadKey ( ) , SetupSpeedCodeTable ) ;
	mov	param,acc			;
	ldz	SetupSpeedCodeTable<<1		;
	rcall	FindByteInArr			;
	cpi	acc,0xFF			; IF Inx <> 0xFF THEN
	breq	SeSrv_1				; BEGIN
	ldz	SpeedCodeTable<<1		;  SerialSpeed := SpeedCodeTable [ Inx ] ;
	add	zl,acc				;
	ldi	acc,0				;
	add	zh,acc				;
	lpm					;
	sts	SerialSpeed,r0			;
	ldi	acc,0				; [UBRRH:UBRRL] := SerialSpeed  ;
	out	UBRRH,acc			;
	out	UBRRL,r0			;
SeSrv_1 :					; END (* IF *) ;
	rcall	ClrScrLCDService		; ClrScrLCDService ( ) ;
	ret					;END (* SetupService *) ;
;-----------------------------------------------------------------------------
EmptyService :					;PROCEDURE EmptyService ( ) ;
;************					;BEGIN (* EmptyService *)
	ret					;END (* EmptyService *) ;
;-----------------------------------------------------------------------------
StandardTTY :					;PROCEDURE StandardTTY ( ) ;
;****************				;VAR
;						; Inx[acc] : BYTE ;
;						;BEGIN (* StandardTTY *)
	mov	acc,TTYCharParam		; IF TTYCharParam < ' ' THEN
	subi	acc,' '				;
	brcc	StaTTY_0			; BEGIN
	ldz	StandTTYCodeTable<<1		;  Inx := FindByteInArr ( TTYCharParam , StandTTYCodeTable ) ;
	mov	param,TTYCharParam		;
	rcall	FindByteInArr			;
	cpi	acc,0xFF			;  IF Inx <> 0xFF THEN
	breq	StaTTY_2			;  BEGIN
	ldz	StandTTYServiceTable<<1		;   RunService ( Inx , StandTTYServiceTable ) ;
	mov	param,acc			;
	rcall	RunService			;
StaTTY_2 :					;  END (* IF *) ;
	ret					;  RETURN ;
						; END (* IF ... *)
StaTTY_0 :					; ELSE
						; BEGIN
	mov	param,TTYCharParam		;  WriteChLCD ( TTYCharParam ) ;
	rcall	WriteChLCD			;
						; END (* IF ... ELSE *) ;
	ret					;END (* StandardTTY *) ;
;-----------------------------------------------------------------------------
LeaveControl	:				;PROCEDURE LeaveControl ( ) ;
;****************				;BEGIN (* LeaveControl *)
	ldz	StandardTTY			; TTYService := StandardTTY ;
	mov	TTYServiceL,ZL			;
	mov	TTYServiceH,ZH			;
	ret					;END (* LeaveControl *) ;
;-----------------------------------------------------------------------------
SetTTYService	:				;PROCEDURE SetTTYService ( Serv[Z] : ADDRESS ) ;
;****************				;BEGIN (* SetTTYService *)
	mov	TTYServiceL,ZL			;
	mov	TTYServiceH,ZH			;
	ret					;END (* SetTTYService *) 
;-----------------------------------------------------------------------------
HomeLCDService	:				;PROCEDURE HomeLCDService ( ) ;
;****************				;BEGIN (* HomeLCDService *)
	rcall	HomeLCD				; HomeLCD ( ) ;
	rcall	LeaveControl			; LeaveControl ( ) ;
	ldi	acc,0				; SecondLine := FALSE ;
	sts	SecondLine,acc			;
	ret					;END (* HomeLCDService *) ;
;-----------------------------------------------------------------------------
ClrScrLCDService :				;PROCEDURE ClrScrLCDService ( ) ;
;*****************				;BEGIN (* ClrScrLCDService *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	rcall	LeaveControl			; LeaveControl ( ) ;
	ldi	acc,0				; SecondLine := FALSE ;
	sts	SecondLine,acc			;
	ret					;END (* ClrScrLCDService *) ;
;-----------------------------------------------------------------------------
SetEscControlService :				;PROCEDURE SetEscControlService ( ) ;
;*********************				;BEGIN (* SetEscControlService *)
	ldz	ESCControlService		; SetTTYService ( ESCControlService ) ;
	rcall	SetTTYService			;
	ret					;END (* SetEscControlService *) ;
;-----------------------------------------------------------------------------
ESCControlService :				;PROCEDURE ESCControlService ( ) ;
;******************				;VAR
;						; Inx[acc] : BYTE ;
;						;BEGIN (* ESCControlService *)
	ldz	EscCodeTable<<1			; Inx := FindByteInArr ( TTYCharParam , EscCodeTable ) ;
	mov	param,TTYCharParam		;
	rcall	FindByteInArr			;
	cpi	acc,0xFF			; IF Inx <> 0xFF THEN
	breq	EscCO_0				; BEGIN
	ldz	EscCodeService<<1		;  RunService ( Inx , EscCodeService ) ;
	mov	param,acc			;
	rcall	RunService			;
	ret					;  RETURN ;
EscCO_0	:					; END (* IF *) ;
	rcall	LeaveControl			; LeaveControl ( ) ;
	ret					;END (* ESCControlService *) ;
;-----------------------------------------------------------------------------
NewLineService :				;PROCEDURE NewLineService ( ) ;
;***************				;BEGIN (* NewLineService *)
	rcall	NewLineLCD			; NewLineLCD ( ) ;
	rcall	LeaveControl			; LeaveControl ( ) ;
	ldi	acc,1				; SecondLine := TRUE ;
	sts	SecondLine,acc			;
	ret					;END (* NewLineService *) ;
;-----------------------------------------------------------------------------
HomeLineService :				;PROCEDURE HomeLineService ( ) ;
;****************				;BEGIN (* HomeLineService *)
	lds	acc,SecondLine			; IF SecondLine THEN
	cpi	acc,0				;
	breq	HoLiSe_0			; BEGIN
	ldi	param,0				;  SetCursorPos ( 0 , 1 ) ;
	ldi	acc,1				;
	rcall	SetCursorPos			;
	rjmp	HoLiSe_1			; END (* IF ... *)
HoLiSe_0 :					; ELSE
						; BEGIN
	ldi	param,0				;  SetCursorPos ( 0 , 0 ) ;
	ldi	acc,0				;
	rcall	SetCursorPos			;
HoLiSe_1 :					; END (* IF ... ELSE *) ;
	rcall	LeaveControl			; LeaveControl ( ) ;
	ret					;END (* HomeLineService *) ;
;-----------------------------------------------------------------------------
CursorService	:				;PROCEDURE CursorService ( ) ;
;****************				;BEGIN (* CursorService *)
	mov	acc,TTYCharParam		; IF TTYCharParam = '0' THEN
	cpi	acc,'0'				;
	brne	CursS_0				; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	rjmp	CursS_1				;
						; END (* IF ... *) ;
CursS_0	:					; ELSE
						; BEGIN
	cpi	acc,'1'				;  IF TTYCharParam = '1' THEN
	brne	CursS_2				;  BEGIN
	rcall	SetCursonOn			;   SetCursonOn ( ) ;
CursS_2	:					;  END (* IF *) ;
CursS_1	:					; END (* IF ... ELSE*) ;
	rcall	LeaveControl			; LeaveControl ( ) ;
	ret					;END (* CursorService *) ;
;-----------------------------------------------------------------------------
CursorSelectionService	:			;PROCEDURE CursorSelectionService ( ) ;
;************************			;BEGIN (* CursorSelectionService *)
	ldz	CursorService			; SetTTYService ( CursorService ) ;
	rcall	SetTTYService			;
	ret					;END (* CursorSelectionService *) ;
;-----------------------------------------------------------------------------
GetColumnSetCursorService :			;PROCEDURE GetColumnSetCursorService ( ) ;
;*****************				;BEGIN (* GetColumnSetCursorService *)
	mov	acc,TTYCharParam		; IF TTYCharParam = ':' THEN
	cpi	acc,':'				;
	brne	SePaSetC_0			; BEGIN
	ldz	GetRowSetCursorService		;  SetTTYService ( GetRowSetCursorService ) ;
	rcall	SetTTYService			;
	ret					;  RETURN ;
SePaSetC_0 :					; END (* IF *) ;
	cpi	acc,'0'				; IF TTYCharParam < '0' THEN
	brcc	SePaSetC_1			; BEGIN
	rcall	LeaveControl			;  LeaveControl ( ) ;
	ret					;  RETURN ;
SePaSetC_1 :					; END (* IF *) ;
	cpi	acc,'9'+1			; IF TTYCharParam > '9' THEN
	brcs	SePaSetC_2			; BEGIN
	rcall	LeaveControl			;  LeaveControl ( ) ;
	ret					;  RETURN ;
SePaSetC_2 :					; END (* IF *) ;
	ldz	Col				; Col := Col * 10 + ( TTYCharParam - '0' ) ;
	ld	acc2,z				;
	lsl	acc2				;
	lsl	acc2				;
	lsl	acc2				;
	ld	acc,z				;
	lsl	acc				;
	add	acc2,acc			;
	add	acc2,TTYCharParam		;
	subi	acc2,'0'			;
	st	z,acc2				;
	ret					;END (* GetColumnSetCursorService *) ;
;-----------------------------------------------------------------------------
GetRowSetCursorService :			;PROCEDURE GetRowSetCursorService ( ) ;
;*****************				;BEGIN (* GetRowSetCursorService *)
	mov	acc,TTYCharParam		; IF ( TTYCharParam = '0' ) OR ( TTYCharParam = '1' ) THEN
	cpi	acc,'0'				;
	breq	SePaSetR_1			;
	cpi	acc,'1'				;
	brne	SePaSetR_2			; BEGIN
SePaSetR_1 :					;
	lds	param,Col			;  SetCursorPos ( Col , TTYCharParam - '0' ) ;
	mov	acc,TTYCharParam		;
	subi	acc,'0'				;
	rcall	SetCursorPos			;
SePaSetR_2 :					; END (* IF *) ;
	rcall	LeaveControl			; LeaveControl ( ) ;
	ret					;END (* GetRowSetCursorService *) ;
;-----------------------------------------------------------------------------
SetCursorPosService :				;PROCEDURE SetCursorPosService ( ) ;
;**************************			;BEGIN (* SetCursorPosService *)
	ldz	GetColumnSetCursorService	; SetTTYService ( GetColumnSetCursorService ) ;
	rcall	SetTTYService			;
	ldi	acc,0				; Col := 0 ;
	sts	Col,acc				;
	ret					;END (* SetCursorPosService *) ;
;-----------------------------------------------------------------------------
KeyboardService :				;PROCEDURE KeyboardService ( ) ;
;****************				;VAR
;						; Inx[acc] : BYTE ;
;						;BEGIN (* KeyboardService *)
	rcall	ReadKey				; Inx := FindByteInArr ( ReadKey ( ) , KeybCodeTable ) ;
	mov	param,acc			;
	ldz	KeybCodeTable<<1		;
	rcall	FindByteInArr			;
	cpi	acc,0xFF			; IF Inx <> 0xFF THEN
	breq	KbdSe_0				; BEGIN
	ldz	KeybServiceTable<<1		;  RunService ( Inx , KeybServiceTable ) ;
	mov	param,acc			;
	rcall	RunService			;
	ret					;  RETURN ;
KbdSe_0	:					; END (* IF *) ;
	ret					;END (* KeyboardService *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
Main :						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	InitialiseLCD			; InitialiseLCD ( ) ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	Hello<<1			; WriteTextLCDFlash ( Hello ) ;
	rcall	WriteTextLCDFlash		;
	rcall	LeaveControl			; LeaveControl ( ) ;
Main_0	:					; LOOP
	rcall	SerialDataPresent		;  IF SerialDataPresent ( ) THEN
	brcc	Main_1				;  BEGIN
	lds	acc,Idle			;   IF Idle THEN
	cpi	acc,0				;
	breq	Main_2				;   BEGIN
	ldi	acc,0				;
	sts	Idle,acc			;    Idle := FALSE ;
	rcall	ClrScrLCD			;    ClrScrLCD ( ) ;
Main_2	:					;   END (* IF *) ;
	rcall	GetSerial			;   TTYCharParam := GetSerial ( ) ;
	mov	TTYCharParam,acc		;
	mov	zl,TTYServiceL			;   TTYService ( ) ;
	mov	zh,TTYServiceH			;
	icall					;
Main_1	:					;  END (* IF *) ;
	rcall	KeyPressed			;  IF KeyPressed ( ) THEN
	brcc	Main_3				;  BEGIN
	rcall	KeyboardService			;   KeyboardService ( ) ;
Main_3	:					;  END (* IF *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *) ;
;-----------------------------------------------------------------------------
Hello :			.db	"Hello, tu 'Sztuka programowania'",0,0
AnyKeyTxt :		.db	"Wcisnij dowolny klawisz",0
SerialMode :		.db	"N81/",0,0
SerialParamPrompt :	.db	"Parametry transmisji: ",0,0
UnknownTxt :		.db	"???",0
SetupMenuTxt :		.db	"Zmiana predk.: F1-2400 F2-4800 F3-9600",0,0
SetupMenuTxt2 :		.db	"F4-19200 F5-38400 F6-115200 F8-zaniech.",0
Speed2400Txt :		.db	"2400",0,0
Speed4800Txt :		.db	"4800",0,0
Speed9600Txt :		.db	"9600",0,0
Speed19200Txt :		.db	"19200",0
Speed38400Txt :		.db	"38400",0
Speed115200Txt :	.db	"115200",0,0
;-----------------------------------------------------------------------------
SetupSpeedCodeTable :	.db	Key0Code , Key1Code	; inx=0  , inx=1
			.db	Key2Code , Key3Code	; inx=2  , inx=3
			.db	Key4Code , Key5Code	; inx=4  , inx=5
			.db	0xFF     , 0xFF
SpeedCodeTable :	.db	UBRRForSpeed2400  , UBRRForSpeed4800	; inx=0  , inx=1
			.db	UBRRForSpeed9600  , UBRRForSpeed19200	; inx=2  , inx=3
			.db	UBRRForSpeed38400 , UBRRForSpeed115200	; inx=4  , inx=5
			.db	0xFF	, 0xFF		;
DispSpeedServiceTable :	.dw	Speed2400Txt<<1		; inx=0
			.dw	Speed4800Txt<<1		; inx=1
			.dw	Speed9600Txt<<1		; inx=2
			.dw	Speed19200Txt<<1	; inx=3
			.dw	Speed38400Txt<<1	; inx=4
			.dw	Speed115200Txt<<1	; inx=5
KeybCodeTable :		.db	Key0Code , Key1Code	; inx=0  , inx=1
			.db	Key2Code , Key3Code	; inx=2  , inx=3
			.db	Key4Code , Key5Code	; inx=4  , inx=5
			.db	Key6Code , Key7Code	; inx=6  , inx=7
			.db	0xFF     , 0xFF
KeybServiceTable :	.dw	ClrScrLCDService	; inx=0
			.dw	DisplaySerialParam	; inx=1
			.dw	SetupService		; inx=2
			.dw	EmptyService		; inx=3
			.dw	EmptyService		; inx=4
			.dw	EmptyService		; inx=5
			.dw	EmptyService		; inx=6
			.dw	EmptyService		; inx=7
;-----------------------------------------------------------------------------
StandTTYCodeTable :	.db	Cr  , Lf		; inx=0  , inx=1
			.db	Esc , 0xFF		; inx=2
StandTTYServiceTable :	.dw	HomeLineService		; inx=0
			.dw	NewLineService		; inx=1
			.dw	SetEscControlService	; inx=2
EscCodeTable :		.db	'H'  , 'J'		; inx=0  , inx=1
			.db	'C'  , 'Y'		; inx=2  , inx=3
			.db	0xFF , 0xFF		;
EscCodeService :	.dw	HomeLCDService		; inx=0
			.dw	ClrScrLCDService	; inx=1
			.dw	CursorSelectionService	; inx=2
			.dw	SetCursorPosService	; inx=3
;-----------------------------------------------------------------------------
.exit
