;
;*******************************************************
;*******************************************************
;**                                                   **
;**      Program prezentacji obsugi modulu LCD       **
;** modul LCD 16 znakow jako 2*8, interfejs 4-bitowy  **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def acc		= r16
.def acc2		= r17
.def param		= r18
;-----------------------------------------------------------------------------
.equ	TimerData	= 0xA0
.equ	DivideConst	= 50
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000100
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : LCD D0
;					  ||||||+--- output : LCD D1
;					  |||||+---- output : LCD D2
;					  ||||+----- output : LCD D3
;					  |||+------ output : LCD RS
;					  ||+------- output : LCD RW
;					  |+-------- output : LCD E
;					  +--------- output :
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output :
;					  ||||||+--- output :
;					  |||||+---- output :
;					  ||||+----- output :
;					  |||+------ output :
;					  ||+------- output :
;					  |+-------- output :
;					  +--------- output :
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output :
;					  ||||||+--- output :
;					  |||||+---- output :
;					  ||||+----- output :
;					  |||+------ output :
;					  ||+------- output :
;					  |+-------- output :
;					  +--------- output :
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input :
;					  ||||||+--- input :
;					  |||||+---- input :
;					  ||||+----- input :
;					  |||+------ input :
;					  ||+------- input :
;					  |+-------- input :
;					  +--------- input :
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						; VAR
TimeDivider :		.byte	1		;  TimeDivider : BYTE ;
MessageIndex :		.byte	1		;  MessageIndex : BYTE ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.equ	V1x16_as_2x8	= 1
.equ	lcd_Interf4b	= 1
.equ	lcd_E_port	= PORTA
.equ	lcd_E_pin	= 6
.equ	lcd_RW_port	= PORTA
.equ	lcd_RW_pin	= 5
.equ	lcd_RS_port	= PORTA
.equ	lcd_RS_pin	= 4
.equ	lcdD0		= 0
.equ	lcdD1		= 1
.equ	lcdD2		= 2
.equ	lcdD3		= 3
.equ	lcd_Data_port	= PORTA
.include "lcdlib.inc"
.equ	LCD_Size	= 16
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	rcall	InitLCDEnvir			; InitLCDEnvir ( ) ;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	ldi	acc,0				; TimeDivider := 0 ;
	sts	TimeDivider,acc			;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	ldz	TimeDivider			; TimeDivider := TimeDivider + 1 ;
	ld	acc,Z				;
	inc	acc				;
	st	Z,acc				;
	cpi	acc,DivideConst			; IF TimeDivider = DivideConst THEN
	brne	TC0OV_0				; BEGIN
	ldi	acc,0				;  TimeDivider := 0 ;
	st	Z,acc				;
TC0OV_0		:				; END (* IF *) ;
	ldi	acc,TimerData			; TCNT0 := TimerData ;
	out	TCNT0,acc			;
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
HelloMessage	:				;PROCEDURE HelloMessage ( ) ;
;						; VAR
;						;  ChPtr[z] : POINTER TO CHAR ;
;						;  Inx[acc2] : BYTE ;
;						;  Ch[param] : CHAR ;
;****************				;BEGIN (* HelloMessage *)
	ldi	acc,0				; MessageIndex := 0 ;
	sts	MessageIndex,acc		;
HeMsg_0	:					; LOOP
	ldz	HelloTxt<<1			;  ChPtr := ADR ( HelloTxt [ MessageIndex ] ) ;
	lds	acc,MessageIndex		;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;  IF ChPtr ^ = 0 THEN
	mov	acc,r0				;
	cpi	acc,0				;
	brne	HeMsg_4				;  BEGIN
	ldi	acc,0				;   MessageIndex := 0 ;
	sts	MessageIndex,acc		;
	ldz	HelloTxt<<1			;   ChPtr := ADR ( HelloTxt [ 0 ] ) ;
HeMsg_4	:					;  END (* IF *) ;
	rcall	ClrScrLCD			;  ClrScrLCD ( ) ;
	ldi	acc2,1				;  FOR Inx := 1 TO LCD_Size DO
HeMsg_3	:					;  BEGIN
	lpm					;   Ch := ChPtr ^ ;
	mov	param,r0			;
	adiw	zl,1				;   ChPtr := ChPtr + 1 ;
	cpi	param,0				;   IF Ch = 0 THEN
	brne	HeMsg_1				;   BEGIN
	ldz	HelloTxt<<1			;    ChPtr := ADR ( HelloTxt [ 0 ] ) ;
	lpm					;    Ch := ChPtr ^ ;
	mov	param,r0			;
	adiw	zl,1				;    ChPtr := ChPtr + 1 ;
HeMsg_1	:					;   END (* IF *) ;
	push	acc2				;
	rcall	WriteChLCD			;   WriteChLCD ( Ch ) ;
	pop	acc2				;
	cpi	acc2,LCD_Size			;
	breq	HeMsg_2				;
	inc	acc2				;
	rjmp	HeMsg_3				;
HeMsg_2	:					;  END (* FOR *) ;
	ldz	MessageIndex			;  MessageIndex := MessageIndex + 1 ;
	ld	acc,z				;
	inc	acc				;
	st	z,acc				;
	rcall	LongWait			;  LongWait ( ) ;
	rjmp	HeMsg_0				; END (* LOOP *) ;
	ret					;END (* HelloMessage *) ;
;-----------------------------------------------------------------------------
LongWait	:				;PROCEDURE LongWait ( ) ;
;****************				;BEGIN
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	ret					;END (* LongWait *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
	rcall	LongWait			; LongWait ( ) ;
	rcall	LongWait			; LongWait ( ) ;
	rcall	InitialiseLCD			; InitialiseLCD ( ) ;
	rcall	HelloMessage			; HelloMessage ( ) ;
Main_0	:					; LOOP
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *) ;
;-----------------------------------------------------------------------------
HelloTxt:	.db	"*** Czesc, tu Twoj procek ATMEGA8515"
		.db	" z modulem LCD 1*16 jako 2*8 znakow,"
		.db	" interfejs 4-bitowy  ***   " , 0
.exit
