;
;*******************************************************
;*******************************************************
;**                                                   **
;**      Program zegara z ukladem RTC (I2I)           **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def acc		= r16
.def acc2		= r17
.def param		= r18
;-----------------------------------------------------------------------------
.equ	DivideConst	= 0x80
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : LCD D0
;					  ||||||+--- output : LCD D1
;					  |||||+---- output : LCD D2
;					  ||||+----- output : LCD D3
;					  |||+------ output : LCD RS
;					  ||+------- output : LCD RW
;					  |+-------- output : LCD E
;					  +--------- output :
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output :
;					  ||||||+--- output :
;					  |||||+---- output :
;					  ||||+----- output :
;					  |||+------ output :
;					  ||+------- output :
;					  |+-------- output :
;					  +--------- output :
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : KEY
;					  ||||||+--- input : KEY
;					  |||||+---- input : KEY
;					  ||||+----- input : KEY
;					  |||+------ input : KEY
;					  ||+------- input : KEY
;					  |+-------- input : KEY
;					  +--------- input : KEY
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output :
;					  ||||||+--- output :
;					  |||||+---- output :
;					  ||||+----- output :
;					  |||+------ output : SDA
;					  ||+------- output : SCL
;					  |+-------- output :
;					  +--------- output :
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
.equ	Key0Pin		= 0
.equ	Key1Pin		= 1
.equ	Key2Pin		= 2
.equ	Key3Pin		= 3
.equ	Key4Pin		= 4
.equ	Key5Pin		= 5
.equ	Key6Pin		= 6
.equ	Key7Pin		= 7
;-----------------------------------------------------------------------------
.equ	SetTimeKey	= 0x20
.equ	SetDateKey	= 0x21
.equ	Char_0Key	= 0x80
.equ	Char_1Key	= 0x81
.equ	Char_2Key	= 0x82
.equ	Char_3Key	= 0x83
.equ	Char_4Key	= 0x84
.equ	Char_5Key	= 0x85
.equ	Char_6Key	= 0x86
.equ	Char_7Key	= 0x87
.equ	Char_8Key	= 0x88
.equ	Char_9Key	= 0x89
.equ	Char_Bs		= 0x22
.equ	Char_Esc	= 0x23
;-----------------------------------------------------------------------------
.equ	Key0Code	= Char_5Key
.equ	Key1Code	= Char_4Key
.equ	Key2Code	= Char_3Key
.equ	Key3Code	= Char_2Key
.equ	Key4Code	= Char_1Key
.equ	Key5Code	= Char_Bs
.equ	Key6Code	= Char_Esc
.equ	Key7Code	= Char_0Key
.equ	Key8Code	= Char_9Key
.equ	Key9Code	= Char_8Key
.equ	Key10Code	= Char_7Key
.equ	Key11Code	= Char_6Key
.equ	Key12Code	= SetTimeKey
.equ	Key13Code	= SetDateKey
;-----------------------------------------------------------------------------
.equ	TimeDivider	= 0x20
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						; CONST
.equ	RTC_Address	= 0xA2			;  RTC_Address = 0xA0 ;
.equ	RTC_SecOffset	= 0x02			;  RTC_SecOffset = 0x02 ;
.equ	RTC_MinOffset	= 0x03			;  RTC_MinOffset = 0x03 ;
.equ	RTC_HourOffset	= 0x04			;  RTC_HourOffset = 0x04 ;
.equ	RTC_DayOffset	= 0x05			;  RTC_DayOffset = 0x05 ;
.equ	RTC_MonthOffset	= 0x07			;  RTC_MonthOffset = 0x07 ;
.equ	RTC_YearOffset	= 0x08			;  RTC_YearOffset = 0x08 ;
.equ	PromptSize	= 6			;  PromptSize = 6 ;
;						;
;						; VAR
LastTimerSec:	.byte	1			;  LastTimerSec : BYTE ;
LastTimerDay:	.byte	1			;  LastTimerDay : BYTE ;
TimerIRQCnt:	.byte	1			;  TimerIRQCnt : BYTE ;
TimerSec:	.byte	1			;  TimerSec : BYTE ;
TimerMin:	.byte	1			;  TimerMin : BYTE ;
TimerHour:	.byte	1			;  TimerHour : BYTE ;
TimerDay:	.byte	1			;  TimerDay : BYTE ;
TimerMonth:	.byte	1			;  TimerMonth : BYTE ;
TimerYear:	.byte	1			;  TimerYear : BYTE ;
TimeDisplMode:	.byte	1			;  TimeDisplMode : BOOLEAN ;
TimeEvent:	.byte	1			;  TimeEvent : BOOLEAN ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.equ	lcd_as1line	= 1
.equ	lcd_Interf4b	= 1
.equ	lcd_E_port	= PORTA
.equ	lcd_E_pin	= 6
.equ	lcd_RW_port	= PORTA
.equ	lcd_RW_pin	= 5
.equ	lcd_RS_port	= PORTA
.equ	lcd_RS_pin	= 4
.equ	lcdD0		= 0
.equ	lcdD1		= 1
.equ	lcdD2		= 2
.equ	lcdD3		= 3
.equ	lcd_Data_port	= PORTA
.include "lcdlib.inc"
;-----------------------------------------------------------------------------
.equ	SDA_Port		= PORTD
.equ	SCL_Port		= PORTD
.equ	SDA_Pin			= 4
.equ	SCL_Pin			= 5
.include "i2clib.inc"
;-----------------------------------------------------------------------------
.equ	KeyPort		= PORTC
.equ	KeyPDirection	= KeyPort - 1
.equ	KeyPInput	= KeyPort - 2
.include "keybsrv.inc"
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	rcall	InitLCDEnvir			; InitLCDEnvir ( ) ;
	rcall	I2CInitEnvir			; I2CInitEnvir ( ) ;
	ldi	acc,0				; KeyPort := <input> + <pullup>;
	out	KeyPDirection,acc		;
	ldi	acc,0xFF			;
	out	KeyPort,acc			;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	ldi	acc,1				; TimeDisplMode := TRUE ;
	sts	TimeDisplMode,acc		;
	ldi	acc,0				; TimerIRQCnt := 0 ;
	sts	TimerIRQCnt,acc			;
	ldi	acc,0xFF			; LastTimerSec := 0xFF ;
	sts	LastTimerSec,acc		;
	sts	LastTimerDay,acc		; LastTimerDay := 0xFF ;
	rcall	KeybSoftInit			; KeybSoftInit ( ) ;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
AnyKeyActive :					;FUNCTION AnyKeyActive ( ) : BOOLEAN [C] ;
;*************					;BEGIN (* AnyKeyActive *)
	in	acc,KeyPInput			; IF ( KeyPInput AND 0x7F ) = 0x7F THEN
	andi	acc,0x7F			;
	cpi	acc,0x7F			;
	brne	AnKeAct_0			; BEGIN
	clc					;  AnyKeyActive := FALSE ;
	ret					;  RETURN ;
AnKeAct_0 :					; END (* IF *) ;
	sec					; AnyKeyActive := TRUE ;
	ret					;END (* AnyKeyActive *) ;
;-----------------------------------------------------------------------------
DecodeKey :					;PROCEDURE DecodeKey ( ) ;
;**********					;VAR
;						; Kbd [acc2] : BYTE ;
;						; Temp [acc] : BYTE ;
						;BEGIN (* DecodeKey *)
	in	acc2,KeyPInput			; Kbd := KeyPInput ;
	ldi	acc,0				; Temp := 0 ;
	sbrs	acc2,Key7Pin			; IF Kbd[Key7Pin] THEN (* if shift *)
	rjmp	DecoKe_0			; BEGIN
	sbrs	acc2,Key0Pin			;  IF Kbd[Key0Pin] THEN
	ldi	acc,Key0Code			;   Temp := Key0Code ;
	sbrs	acc2,Key1Pin			;  IF Kbd[Key1Pin] THEN
	ldi	acc,Key1Code			;   Temp := Key1Code ;
	sbrs	acc2,Key2Pin			;  IF Kbd[Key2Pin] THEN
	ldi	acc,Key2Code			;   Temp := Key2Code ;
	sbrs	acc2,Key3Pin			;  IF Kbd[Key3Pin] THEN
	ldi	acc,Key3Code			;   Temp := Key3Code ;
	sbrs	acc2,Key4Pin			;  IF Kbd[Key4Pin] THEN
	ldi	acc,Key4Code			;   Temp := Key4Code ;
	sbrs	acc2,Key5Pin			;  IF Kbd[Key5Pin] THEN
	ldi	acc,Key5Code			;   Temp := Key5Code ;
	sbrs	acc2,Key6Pin			;  IF Kbd[Key6Pin] THEN
	ldi	acc,Key6Code			;   Temp := Key6Code ;
	rjmp	DecoKe_1			; END (* IF ... *)
						; ELSE
DecoKe_0 :					; BEGIN
	sbrs	acc2,Key0Pin			;  IF Kbd[Key0Pin] THEN
	ldi	acc,Key7Code			;   Temp := Key7Code ;
	sbrs	acc2,Key1Pin			;  IF Kbd[Key1Pin] THEN
	ldi	acc,Key8Code			;   Temp := Key8Code ;
	sbrs	acc2,Key2Pin			;  IF Kbd[Key2Pin] THEN
	ldi	acc,Key9Code			;   Temp := Key9Code ;
	sbrs	acc2,Key3Pin			;  IF Kbd[Key3Pin] THEN
	ldi	acc,Key10Code			;   Temp := Key10Code ;
	sbrs	acc2,Key4Pin			;  IF Kbd[Key4Pin] THEN
	ldi	acc,Key11Code			;   Temp := Key11Code ;
	sbrs	acc2,Key5Pin			;  IF Kbd[Key5Pin] THEN
	ldi	acc,Key12Code			;   Temp := Key12Code ;
	sbrs	acc2,Key6Pin			;  IF Kbd[Key6Pin] THEN
	ldi	acc,Key13Code			;   Temp := Key13Code ;
DecoKe_1 :					; END (* IF ... ELSE *)
	rcall	StoreKey			; StoreKey ( Temp ) ;
	ret					;END (* DecodeKey *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	rcall	KeybService			; KeybService ( ) ;
	ldz	TimerIRQCnt			; TimerIRQCnt := TimerIRQCnt + 1 ;
	ld	acc,z				;
	inc	acc				;
	st	z,acc				;
	cpi	acc,TimeDivider			; IF TimerIRQCnt = TimeDivider THEN
	brne	T0OVL_0				; BEGIN
	ldi	acc,0				;  TimerIRQCnt := 0 ;
	st	z,acc				;
	ldi	acc,1				;  TimeEvent := TRUE ;
	sts	TimeEvent,acc			;
T0OVL_0	:					; END (* IF *) ;
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
GetRTCData :					;PROCEDURE GetRTCData ( Ofs [param] : BYTE ) : BYTE ;
;***********					;BEGIN (* GetRTCData *)
	push	param				;
	rcall	StartI2C			; StartI2C ( ) ;
	rcall	I2CDelay			; I2CDelay ( ) ;
	ldi	param,RTC_Address		; OutByte ( RTC_Address ) ;
	rcall	OutByte				;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	GetACK				; ? := GetACK ( ) ;
	pop	param				; OutByte ( Ofs ) ;
	rcall	OutByte				;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	GetACK				; ? := GetACK ( ) ;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	StartI2C			; StartI2C ( ) ;
	rcall	I2CDelay			; I2CDelay ( ) ;
	ldi	param,RTC_Address+1		; OutByte ( RTC_Address + 1 ) ;
	rcall	OutByte				;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	GetACK				; ? := GetACK ( ) ;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	InByte				; acc := InByte ( ) ;
	push	acc				;
	rcall	NotACK				; NotACK ( ) ;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	StopI2C				; StopI2C ( ) ;
	pop	acc				; RETURN acc ;
	ret					;END GetRTCData ;
;-----------------------------------------------------------------------------
SetRTCData :					;PROCEDURE SetRTCData ( Ofs [param] : BYTE ;
;***********								Val [acc] : BYTE ) : BYTE ;
;						;BEGIN (* SetRTCData *)
	cli					; DI ( ) ;
	push	acc				;
	push	param				;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	StartI2C			; StartI2C ( ) ;
	rcall	I2CDelay			; I2CDelay ( ) ;
	ldi	param,RTC_Address		; OutByte ( RTC_Address ) ;
	rcall	OutByte				;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	GetACK				; ? := GetACK ( ) ;
	pop	param				; OutByte ( Ofs ) ;
	rcall	OutByte				;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	GetACK				; ? := GetACK ( ) ;
	pop	param				; OutByte ( Val ) ;
	rcall	OutByte				;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	NotACK				; NotACK ( ) ;
	rcall	I2CDelay			; I2CDelay ( ) ;
	rcall	StopI2C				; StopI2C ( ) ;
	sei					; EI ( ) ;
	ret					;END SetRTCData ;
;-----------------------------------------------------------------------------
GetTime	:					;PROCEDURE GetTime ( ) ;
;********					;BEGIN (* GetTime *)
	ldi	param,RTC_SecOffset		; TimerSec := GetRTCData ( RTC_SecOffset ) bits 6..0 ;
	rcall	GetRTCData			;
	andi	acc,0x7F			;
	sts	TimerSec,acc			;
	ldi	param,RTC_MinOffset		; TimerMin := GetRTCData ( RTC_MinOffset ) bits 6..0 ;
	rcall	GetRTCData			;
	andi	acc,0x7F			;
	sts	TimerMin,acc			;
	ldi	param,RTC_HourOffset		; TimerHour := GetRTCData ( RTC_HourOffset ) bits 5..0 ;
	rcall	GetRTCData			;
	andi	acc,0x3F			;
	sts	TimerHour,acc			;
	ret					;END GetTime ;
;-----------------------------------------------------------------------------
GetDate	:					;PROCEDURE GetDate ( ) ;
;********					;BEGIN (* GetDate *)
	ldi	param,RTC_DayOffset		; TimerDay := GetRTCData ( RTC_DayOffset ) bits 5..0 ;
	rcall	GetRTCData			;
	andi	acc,0x3F			;
	sts	TimerDay,acc			;
	ldi	param,RTC_MonthOffset		; TimerMonth := GetRTCData ( RTC_MonthOffset ) bits 4..0 ;
	rcall	GetRTCData			;
	andi	acc,0x1F			;
	sts	TimerMonth,acc			;
	ldi	param,RTC_YearOffset		; TimerYear := GetRTCData ( RTC_YearOffset ) ;
	rcall	GetRTCData			;
	sts	TimerYear,acc			;
	ret					;END GetDate ;
;-----------------------------------------------------------------------------
SetDate	:					;PROCEDURE SetDate ( ) ;
;********					;BEGIN (* SetDate *)
	ldi	param,RTC_DayOffset		; SetRTCData ( RTC_DayOffset , TimerDay ) ;
	lds	acc,TimerDay			;
	rcall	SetRTCData			;
	ldi	param,RTC_MonthOffset		; SetRTCData ( RTC_MonthOffset , TimerMonth ) ;
	lds	acc,TimerMonth			;
	rcall	SetRTCData			;
	ldi	param,RTC_YearOffset		; SetRTCData ( RTC_YearOffset , TimerYear ) ;
	lds	acc,TimerYear			;
	rcall	SetRTCData			;
	ret					;END SetDate ;
;-----------------------------------------------------------------------------
SetTime	:					;PROCEDURE SetTime ( ) ;
;********					;BEGIN (* SetTime *)
	ldi	param,RTC_SecOffset		; SetRTCData ( RTC_SecOffset , TimerSec ) ;
	lds	acc,TimerSec			;
	rcall	SetRTCData			;
	ldi	param,RTC_MinOffset		; SetRTCData ( RTC_MinOffset , TimerMin ) ;
	lds	acc,TimerMin			;
	rcall	SetRTCData			;
	ldi	param,RTC_HourOffset		; SetRTCData ( RTC_HourOffset , TimerHour ) ;
	lds	acc,TimerHour			;
	rcall	SetRTCData			;
	ret					;END SetTime ;
;-----------------------------------------------------------------------------
BCDToAsciConv :					;PROCEDURE BCDToAsciConv ( BCD[param] : CHAR ) : CHAR[acc] ;
;**************					;BEGIN (* BCDToAsciConv *)
	ldi	acc,'0'				; RETURN ( '0' + ( BCD MOD 16 ) ) ;
	andi	param,0x0F			;
	add	acc,param			;
	ret					;END (* BCDToAsciConv *) ;
;-----------------------------------------------------------------------------
DisplayTime :					;PROCEDURE DisplayTime ( ) ;
;************					;BEGIN (* DisplayTime *)
	ldz	LastTimerSec			; IF LastTimerSec = TimerSec THEN
	ld	acc,z				;
	lds	acc2,TimerSec			;
	cp	acc,acc2			;
	brne	DisTi_0				; BEGIN
	ret					;  RETURN ;
DisTi_0	:					; END (* IF *) ;
	st	z,acc2				; LastTimerSec := TimerSec ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	lds	param,TimerHour			; WriteChLCD ( BCDToAsciConv ( TimerHour.7..4 ) ) ;
	swap	param				;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerHour			; WriteChLCD ( BCDToAsciConv ( TimerHour.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ldi	param,':'			; WriteChLCD ( ':' ) ;
	rcall	WriteChLCD			;
	lds	param,TimerMin			; WriteChLCD ( BCDToAsciConv ( TimerMin.7..4 ) ) ;
	swap	param				;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerMin			; WriteChLCD ( BCDToAsciConv ( TimerMin.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ldi	param,':'			; WriteChLCD ( ':' ) ;
	rcall	WriteChLCD			;
	lds	param,TimerSec			; WriteChLCD ( BCDToAsciConv ( TimerSec.7..4 ) ) ;
	swap	param				;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerSec			; WriteChLCD ( BCDToAsciConv ( TimerSec.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ret					;END (* DisplayTime *) ;
;-----------------------------------------------------------------------------
DisplayDate :					;PROCEDURE DisplayDate ( ) ;
;************					;BEGIN (* DisplayDate *)
	ldz	LastTimerDay			; IF LastTimerDay = TimerDay THEN
	ld	acc,z				;
	lds	acc2,TimerDay			;
	cp	acc,acc2			;
	brne	DisDa_0				; BEGIN
	ret					;  RETURN ;
DisDa_0	:					; END (* IF *) ;
	st	z,acc2				; LastTimerDay := TimerDay ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	lds	param,TimerYear			; WriteChLCD ( BCDToAsciConv ( TimerYear.7..4 ) ) ;
	swap	param				;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerYear			; WriteChLCD ( BCDToAsciConv ( TimerYear.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ldi	param,'.'			; WriteChLCD ( '.' ) ;
	rcall	WriteChLCD			;
	lds	param,TimerMonth		; WriteChLCD ( BCDToAsciConv ( TimerMonth.7..4 ) ) ;
	swap	param				;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerMonth		; WriteChLCD ( BCDToAsciConv ( TimerMonth.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ldi	param,'.'			; WriteChLCD ( '.' ) ;
	rcall	WriteChLCD			;
	lds	param,TimerDay			; WriteChLCD ( BCDToAsciConv ( TimerDay.7..4 ) ) ;
	swap	param				;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerDay			; WriteChLCD ( BCDToAsciConv ( TimerDay.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ret					;END (* DisplayDate *) ;
;-----------------------------------------------------------------------------
GetNextDigit :					;FUNCTION GetNextDigit ( ) : CHAR[acc] ;
;*************					;VAR
;						; Ch[acc] : CHAR ;
;						;BEGIN (* GetNextDigit *)
GeNxCh_1 :					; LOOP
GeNxCh_0 :					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	GeNxCh_0			;
	rcall	ReadKey				;  Ch := ReadKey ( ) ;
	cpi	acc,Char_Bs			;  IF Ch = Char_Bs THEN
	brne	GeNxCh_2			;  BEGIN
						;   GetNextDigit := Ch ;
	ret					;   RETURN ;
GeNxCh_2 :					;  END (* IF *) ;
	cpi	acc,Char_Esc			;  IF Ch = Char_Esc THEN
	brne	GeNxCh_3			;  BEGIN
						;   GetNextDigit := Ch ;
	ret					;   RETURN ;
GeNxCh_3 :					;  END (* IF *) ;
	cpi	acc,Char_0Key			;  IF ( Ch >= Char_0Key ) AND ( Ch <= Char_9Key ) THEN
	brcs	GeNxCh_4			;
	cpi	acc,Char_9Key+1			;
	brcc	GeNxCh_4			;  BEGIN
	subi	acc,Char_0Key			;   GetNextDigit := Ch - Char_0Key ;
	ret					;   RETURN ;
GeNxCh_4 :					;  END (* IF *) ;
	rjmp	GeNxCh_1			; END (* LOOP *) ;
						;END (* GetNextDigit *) ;
;-----------------------------------------------------------------------------
ConvBCDToBin :					;FUNCTION ConvBCDToBin ( Code[param] : BYTE ) : BYTE[acc] ;
;*************					;BEGIN (* ConvBCDToBin *)
	mov	acc,param			; RETURN ( 10 * Code[7..4] + Code[3..0] ) ;
	swap	acc				;
	andi	acc,0x0F			;
	mov	acc2,acc			;
	lsl	acc				;
	lsl	acc				;
	lsl	acc				;
	lsl	acc2				;
	add	acc,acc2			;
	andi	param,0x0F			;
	add	acc,param			;
	ret					;END (* ConvBCDToBin *) ;
;-----------------------------------------------------------------------------
ClearLine :					;PROCEDURE ClearLine ( ) ;
;**********					;BEGIN (* ClearLine *)
	ldi	param,' '			; WriteChLCD ( ' ' ) ;
	rcall	WriteChLCD			;
	ldi	param,' '			; WriteChLCD ( ' ' ) ;
	rcall	WriteChLCD			;
	ret					;END (* ClearLine *) ;
;-----------------------------------------------------------------------------
SetTimeSrv :					;PROCEDURE SetTimeSrv ( ) ;
;***********					;VAR
;						; Ch[acc] : CHAR ;
;						;BEGIN (* SetTimeSrv *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	SetTimePrompt<<1		; WriteTextLCDFlash ( SetTimePrompt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	SetCursonOn			; SetCursonOn ( ) ;
SeTiSe_2 :					; 1:
SeTiSe_12 :					; 2:
	ldi	param,PromptSize+0		; SetCursorPos ( PromptSize + 0 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeTiSe_0			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeTiSe_0 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeTiSe_1			; BEGIN
	rjmp	SeTiSe_2			;  GOTO 1 ;
SeTiSe_1 :					; END (* IF *) ;
	mov	acc2,acc			; TimerHour := Ch << 4 ;
	swap	acc2				;
	andi	acc2,0xF0			;
	sts	TimerHour,acc2			;
	mov	param,acc			; WriteChLCD ( BCDToAsciConv ( TimerHour.7..4 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
SeTiSe_22 :					; 3 :
	ldi	param,PromptSize+1		; SetCursorPos ( PromptSize + 1 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeTiSe_10			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeTiSe_10 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeTiSe_11			; BEGIN
	ldi	param,PromptSize+0		;  SetCursorPos ( PromptSize + 0 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeTiSe_12			;  GOTO 2 ;
SeTiSe_11 :					; END (* IF *) ;
	push	acc				;
	ldz	TimerHour			; TimerHour := TimerHour + Ch ;
	ld	acc2,z				;
	andi	acc2,0xF0			;
	andi	acc,0x0F			;
	or	acc2,acc			;
	st	z,acc2				;
	pop	param				; WriteChLCD ( BCDToAsciConv ( TimerHour.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerHour			; IF ConvBCDToBin ( TimerHour ) >= 24 THEN
	rcall	ConvBCDToBin			;
	cpi	acc,24				;
	brcs	SeTiSe_13			; BEGIN
	ldi	param,PromptSize+0		;  SetCursorPos ( PromptSize + 0 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeTiSe_2			;  GOTO 1 ;
SeTiSe_13 :					; END (* IF *) ;
	ldi	param,':'			; WriteChLCD ( ':' ) ;
	rcall	WriteChLCD			;
SeTiSe_32 :					; 4 :
	ldi	param,PromptSize+3		; SetCursorPos ( PromptSize + 3 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeTiSe_20			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeTiSe_20 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeTiSe_21			; BEGIN
	ldi	param,PromptSize+1		;  SetCursorPos ( PromptSize + 1 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeTiSe_22			;  GOTO 3 ;
SeTiSe_21 :					; END (* IF *) ;
	mov	acc2,acc			; TimerMin := Ch << 4 ;
	swap	acc2				;
	andi	acc2,0xF0			;
	sts	TimerMin,acc2			;
	mov	param,acc			; WriteChLCD ( BCDToAsciConv ( TimerMin.7..4 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
SeTiSe_42 :					; 5 :
	ldi	param,PromptSize+4		; SetCursorPos ( PromptSize + 4 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeTiSe_30			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeTiSe_30 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeTiSe_31			; BEGIN
	ldi	param,PromptSize+3		;  SetCursorPos ( PromptSize + 3 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeTiSe_32			;  GOTO 4 ;
SeTiSe_31 :					; END (* IF *) ;
	push	acc				;
	ldz	TimerMin			; TimerMin := TimerMin + Ch ;
	ld	acc2,z				;
	andi	acc2,0xF0			;
	andi	acc,0x0F			;
	or	acc2,acc			;
	st	z,acc2				;
	pop	param				; WriteChLCD ( BCDToAsciConv ( TimerMin.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerMin			; IF ConvBCDToBin ( TimerMin ) >= 60 THEN
	rcall	ConvBCDToBin			;
	cpi	acc,60				;
	brcs	SeTiSe_33			; BEGIN
	ldi	param,PromptSize+3		;  SetCursorPos ( PromptSize + 3 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeTiSe_32			;  GOTO 4 ;
SeTiSe_33 :					; END (* IF *) ;
	ldi	param,':'			; WriteChLCD ( ':' ) ;
	rcall	WriteChLCD			;
SeTiSe_52 :					; 6 :
	ldi	param,PromptSize+6		; SetCursorPos ( PromptSize + 6 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeTiSe_40			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeTiSe_40 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeTiSe_41			; BEGIN
	ldi	param,PromptSize+4		;  SetCursorPos ( PromptSize + 4 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rcall	WriteChLCD			;
	rjmp	SeTiSe_42			;  GOTO 5 ;
SeTiSe_41 :					; END (* IF *) ;
	mov	acc2,acc			; TimerSecs := Ch << 4 ;
	swap	acc2				;
	andi	acc2,0xF0			;
	sts	TimerSec,acc2			;
	mov	param,acc			; WriteChLCD ( BCDToAsciConv ( TimerSec.7..4 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ldi	param,PromptSize+7		; SetCursorPos ( PromptSize + 7 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeTiSe_50			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeTiSe_50 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeTiSe_51			; BEGIN
	ldi	param,PromptSize+6		;  SetCursorPos ( PromptSize + 6 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeTiSe_52			;  GOTO 6 ;
SeTiSe_51 :					; END (* IF *) ;
	push	acc				;
	ldz	TimerSec			; TimerSec := TimerSec + Ch ;
	ld	acc2,z				;
	andi	acc2,0xF0			;
	andi	acc,0x0F			;
	or	acc2,acc			;
	st	z,acc2				;
	pop	param				; WriteChLCD ( BCDToAsciConv ( TimerSec.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerSec			; IF ConvBCDToBin ( TimerSec ) >= 60 THEN
	rcall	ConvBCDToBin			;
	cpi	acc,60				;
	brcs	SeTiSe_53			; BEGIN
	ldi	param,PromptSize+6		;  SetCursorPos ( PromptSize + 6 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeTiSe_52			;  GOTO 6 ;
SeTiSe_53 :					; END (* IF *) ;
	rcall	SetCursonOff			; SetCursonOff ( ) ;
	rcall	SetTime				; SetTime ( ) ;
	ret					;END (* SetTimeSrv *) ;
;-----------------------------------------------------------------------------
DateFormatOK :					;FUNCTION DateFormatOK ( ) : BOOLEAN ;
;*************					;VAR
;						; DayNumber[acc2] : BYTE ;
;						;BEGIN (* DateFormatOK *)
	lds	param,TimerDay			; DayNumber := ConvBCDToBin ( TimerDay ) ;
	rcall	ConvBCDToBin			;
	mov	acc2,acc			;
	cpi	acc2,0				; IF DayNumber = 0 THEN
	brne	DaFoOK_0			; BEGIN
	clc					;  DateFormatOK := FALSE ;
	ret					;  RETURN ;
DaFoOK_0 :					; END (* IF *) ;
	lds	acc,TimerMonth			; IF TimerMonth = 2 THEN
	cpi	acc,2				;
	brne	DaFoOK_1			; BEGIN
	lds	param,TimerYear			;  IF ( ConvBCDToBin ( TimerYear ) MOD 4 ) = 0 THEN
	push	acc2				;
	rcall	ConvBCDToBin			;
	pop	acc2				;
	andi	acc,0x03			;
	cpi	acc,0				;
	brne	DaFoOK_4			;  BEGIN
	cpi	acc2,30				;   DateFormatOK ( DayNumber < 30 ) ;
	ret					;   RETURN ;
DaFoOK_4 :					;  END (* IF *) ;
	cpi	acc2,29				;  DateFormatOK ( DayNumber < 29 ) ;
	ret					;  RETURN ;
DaFoOK_1 :					; END (* IF *) ;
	cpi	acc,1				; IF ( TimerMonth = 1 ) OR
	breq	DaFoOK_2			;
	cpi	acc,3				;    ( TimerMonth = 3 ) OR
	breq	DaFoOK_2			;
	cpi	acc,5				;    ( TimerMonth = 5 ) OR
	breq	DaFoOK_2			;
	cpi	acc,7				;    ( TimerMonth = 7 ) OR
	breq	DaFoOK_2			;
	cpi	acc,8				;    ( TimerMonth = 8 ) OR
	breq	DaFoOK_2			;
	cpi	acc,10				;    ( TimerMonth = 10 ) OR
	breq	DaFoOK_2			;
	cpi	acc,12				;    ( TimerMonth = 12 ) THEN
	brne	DaFoOK_3			;
DaFoOK_2 :					; BEGIN
	cpi	acc2,32				;  DateFormatOK ( DayNumber < 32 ) ;
	ret					;  RETURN ;
DaFoOK_3 :					; END (* IF *) ;
	cpi	acc2,31				; DateFormatOK ( DayNumber < 31 ) ;
	ret					;END (* DateFormatOK *) ;
;-----------------------------------------------------------------------------
SetDateSrv :					;PROCEDURE SetDateSrv ( ) ;
;***********					;VAR
;						; Ch[acc] : CHAR ;
;						;BEGIN (* SetDateSrv *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	SetDatePrompt<<1		; WriteTextLCDFlash ( SetDatePrompt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	SetCursonOn			; SetCursonOn ( ) ;
SeDaSe_2 :					; 1:
SeDaSe_12 :					; 2:
	ldi	param,PromptSize+0		; SetCursorPos ( PromptSize + 0 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeDaSe_0			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeDaSe_0 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeDaSe_1			; BEGIN
	rjmp	SeDaSe_2			;  GOTO 1 ;
SeDaSe_1 :					; END (* IF *) ;
	mov	acc2,acc			; TimerYear := Ch << 4 ;
	swap	acc2				;
	andi	acc2,0xF0			;
	sts	TimerYear,acc2			;
	mov	param,acc			; WriteChLCD ( BCDToAsciConv ( TimerYear.7..4 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
SeDaSe_22 :					; 3 :
	ldi	param,PromptSize+1		; SetCursorPos ( PromptSize + 1 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeDaSe_10			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeDaSe_10 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeDaSe_11			; BEGIN
	ldi	param,PromptSize+0		;  SetCursorPos ( PromptSize + 0 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeDaSe_12			;  GOTO 2 ;
SeDaSe_11 :					; END (* IF *) ;
	push	acc				;
	ldz	TimerYear			; TimerYear := TimerYear + Ch ;
	ld	acc2,z				;
	andi	acc2,0xF0			;
	andi	acc,0x0F			;
	or	acc2,acc			;
	st	z,acc2				;
	pop	param				; WriteChLCD ( BCDToAsciConv ( TimerYear.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ldi	param,'.'			; WriteChLCD ( '.' ) ;
	rcall	WriteChLCD			;
SeDaSe_32 :					; 4 :
	ldi	param,PromptSize+3		; SetCursorPos ( PromptSize + 3 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeDaSe_20			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeDaSe_20 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeDaSe_21			; BEGIN
	ldi	param,PromptSize+1		;  SetCursorPos ( PromptSize + 1 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeDaSe_22			;  GOTO 3 ;
SeDaSe_21 :					; END (* IF *) ;
	mov	acc2,acc			; TimerMonth := Ch << 4 ;
	swap	acc2				;
	andi	acc2,0xF0			;
	sts	TimerMonth,acc2			;
	mov	param,acc			; WriteChLCD ( BCDToAsciConv ( TimerMonth.7..4 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
SeDaSe_42 :					; 5 :
	ldi	param,PromptSize+4		; SetCursorPos ( PromptSize + 4 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeDaSe_30			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeDaSe_30 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeDaSe_31			; BEGIN
	ldi	param,PromptSize+3		;  SetCursorPos ( PromptSize + 3 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeDaSe_32			;  GOTO 4 ;
SeDaSe_31 :					; END (* IF *) ;
	push	acc				;
	ldz	TimerMonth			; TimerMonth := TimerMonth + Ch ;
	ld	acc2,z				;
	andi	acc2,0xF0			;
	andi	acc,0x0F			;
	or	acc2,acc			;
	st	z,acc2				;
	pop	param				; WriteChLCD ( BCDToAsciConv ( TimerMonth.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	lds	param,TimerMonth		; IF ( ConvBCDToBin ( TimerMonth ) >= 13 ) OR ( TimerMonth = 0 ) THEN
	rcall	ConvBCDToBin			;
	cpi	acc,0				;
	breq	SeDaSe_34			;
	cpi	acc,13				;
	brcs	SeDaSe_33			;
SeDaSe_34:					; BEGIN
	ldi	param,PromptSize+3		;  SetCursorPos ( PromptSize + 3 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeDaSe_32			;  GOTO 4 ;
SeDaSe_33 :					; END (* IF *) ;
	ldi	param,'.'			; WriteChLCD ( '.' ) ;
	rcall	WriteChLCD			;
SeDaSe_52 :					; 6 :
	ldi	param,PromptSize+6		; SetCursorPos ( PromptSize + 6 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeDaSe_40			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeDaSe_40 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeDaSe_41			; BEGIN
	ldi	param,PromptSize+4		;  SetCursorPos ( PromptSize + 4 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeDaSe_42			;  GOTO 5 ;
SeDaSe_41 :					; END (* IF *) ;
	mov	acc2,acc			; TimerDays := Ch << 4 ;
	swap	acc2				;
	andi	acc2,0xF0			;
	sts	TimerDay,acc2			;
	mov	param,acc			; WriteChLCD ( BCDToAsciConv ( TimerDay.7..4 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	ldi	param,PromptSize+7		; SetCursorPos ( PromptSize + 7 ) ;
	rcall	SetCursorPos			;
	rcall	GetNextDigit			; Ch := GetNextDigit ( ) ;
	cpi	acc,Char_Esc			; IF Ch = Char_Esc THEN
	brne	SeDaSe_50			; BEGIN
	rcall	SetCursonOff			;  SetCursonOff ( ) ;
	ret					;  RETURN ;
SeDaSe_50 :					; END (* IF *) ;
	cpi	acc,Char_Bs			; IF Ch = Char_Bs THEN
	brne	SeDaSe_51			; BEGIN
	ldi	param,PromptSize+6		;  SetCursorPos ( PromptSize + 6 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeDaSe_52			;  GOTO 6 ;
SeDaSe_51 :					; END (* IF *) ;
	push	acc				;
	ldz	TimerDay			; TimerDay := TimerDay + Ch ;
	ld	acc2,z				;
	andi	acc2,0xF0			;
	andi	acc,0x0F			;
	or	acc2,acc			;
	st	z,acc2				;
	pop	param				; WriteChLCD ( BCDToAsciConv ( TimerDay.3..0 ) ) ;
	rcall	BCDToAsciConv			;
	mov	param,acc			;
	rcall	WriteChLCD			;
	rcall	DateFormatOK			; IF NOT DateFormatOK ( ) THEN
	brcs	SeDaSe_53			;
	ldi	param,PromptSize+6		;  SetCursorPos ( PromptSize + 6 ) ;
	rcall	SetCursorPos			;
	rcall	ClearLine			;  ClearLine ( ) ;
	rjmp	SeDaSe_52			;  GOTO 6 ;
SeDaSe_53 :					; END (* IF *) ;
	rcall	SetCursonOff			; SetCursonOff ( ) ;
	rcall	SetDate				; SetDate ( ) ;
	ret					;END (* SetDateSrv *) ;
;-----------------------------------------------------------------------------
RunCommand :					;PROCEDURE RunCommand ( ) ;
;***********					;VAR
;						; Ch[acc] : CHAR ;
;						;BEGIN (* RunCommand *)
	rcall	ReadKey				; Ch := ReadKey ( ) ;
	cpi	acc,SetTimeKey			; IF Ch = SetTimeKey THEN
	brne	RuCmm_0				; BEGIN
	rcall	SetTimeSrv			;  SetTimeSrv ( ) ;
	ret					;  RETURN ;
RuCmm_0 :					; END (* IF *) ;
	cpi	acc,SetDateKey			; IF Ch = SetDateKey THEN
	brne	RuCmm_1				; BEGIN
	rcall	SetDateSrv			;  SetDateSrv ( ) ;
	ret					;  RETURN ;
RuCmm_1 :					; END (* IF *) ;
	cpi	acc,Char_5Key			; IF Ch = Char_5Key THEN
	brne	RuCmm_2				; BEGIN
	ldi	acc,1				;  TimeDisplMode := TRUE ;
	sts	TimeDisplMode,acc		;
	ldi	acc,0xFF			;  LastTimerSec := 0xFF ;
	sts	LastTimerSec,acc		;
	ret					;  RETURN ;
RuCmm_2	:					; END (* IF *) ;
	cpi	acc,Char_4Key			; IF Ch = Char_4Key THEN
	brne	RuCmm_3				; BEGIN
	ldi	acc,0				;  TimeDisplMode := FALSE ;
	sts	TimeDisplMode,acc		;
	ldi	acc,0xFF			;  LastTimerDay := 0xFF ;
	sts	LastTimerDay,acc		;
	ret					;  RETURN ;
RuCmm_3	:					; END (* IF *) ;
	ret					;END (* RunCommand *) ;
;-----------------------------------------------------------------------------
LongWait	:				;PROCEDURE LongWait ( ) ;
;****************				;BEGIN
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	ret					;END (* LongWait *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	cli					;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	rcall	LongWait			; LongWait ( ) ;
	rcall	InitialiseLCD			; InitialiseLCD ( ) ;
	sei					; EI ( ) ;
Main_0	:					; LOOP
	rcall	KeyPressed			;  IF KeyPressed ( ) THEN
	brcc	Main_3				;  BEGIN
	rcall	RunCommand			;   RunCommand ( ) ;
Main_3	:					;  END (* IF *) ;
	ldz	TimeEvent			;  IF TimeEvent THEN
	ld	acc,z				;
	cpi	acc,0				;
	breq	Main_4				;  BEGIN
	ldi	acc,0				;   TimeEvent := FALSE ;
	st	z,acc				;
	lds	acc,TimeDisplMode		;   IF TimeDisplMode THEN
	cpi	acc,0				;
	breq	Main_1				;   BEGIN
	rcall	GetTime				;    GetTime ( ) ;
	rcall	DisplayTime			;    DisplayTime ( ) ;
	rjmp	Main_2				;   END (* IF ... *)
Main_1	:					;   ELSE
						;   BEGIN
	rcall	GetDate				;    GetDate ( ) ;
	rcall	DisplayDate			;    DisplayDate ( ) ;
Main_2	:					;   END (* IF ... ELSE *) ;
Main_4	:					;  END (* IF *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *) ;
;-----------------------------------------------------------------------------
SetTimePrompt:	.db	"Godz: ",0,0
SetDatePrompt:	.db	"Data: ",0,0
;-----------------------------------------------------------------------------
.exit
