;
;*******************************************************
;*******************************************************
;**                                                   **
;**      Program prezentacji operacji na datach       **
;**                                                   **
;**               Andrzej Pawluczuk                   **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def DivLoopCounter	= r1
.def Argument1LL	= r2
.def Argument1LH	= r3
.def Argument1HL	= r4
.def Argument1HH	= r5
.def Argument2LL	= r6
.def Argument2LH	= r7
.def Argument2HL	= r8
.def Argument2HH	= r9
.def WorkRegLL		= r10
.def WorkRegLH		= r11
.def WorkRegHL		= r12
.def WorkRegHH		= r13
.def acc		= r16
.def acc2		= r17
.def param		= r18
.def ResultLL		= r22
.def ResultLH		= r23
.def ResultHL		= r24
.def ResultHH		= r25
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : 
;					  ||||||+--- input : 
;					  |||||+---- input : 
;					  ||||+----- input : 
;					  |||+------ input : 
;					  ||+------- input : 
;					  |+-------- input : 
;					  +--------- input : 
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output :
;					  +--------- output :
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : 
;					  ||||||+--- input : 
;					  |||||+---- input : 
;					  ||||+----- input : 
;					  |||+------ input : 
;					  ||+------- input : 
;					  |+-------- input : 
;					  +--------- input : 
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11111110
;					  ||||||||
;					  |||||||+-- input :	[RXD]
;					  ||||||+--- output :	[TXD]
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
.equ	Cr		=	0x0D		;  Cr = 0DH ;
.equ	Lf		=	0x0A		;  Lf = 0AH ;
;						;
.equ	TXDBuffSize	=	16		;  TXDBuffSize = 16 ;
.equ	RXDBuffSize	=	8		;  RXDBuffSize = 8 ;
;						;
;						; VAR
DayNo:		.byte	1			;  DayNo : BYTE ;
MonthNo:	.byte	1			;  MonthNo : BYTE ;
YearNo:		.byte	2			;  YearNo : WORD ;
Card:		.byte	2			;  Card : WORD 
Date:		.byte	2			;  Date : WORD ;
Number:		.byte	8			;  Number : ARRAY [ 0 .. 7 ] OF CHAR ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	rjmp	USARTRXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	rjmp	USARTDataRegEmptInterrupt	;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	rjmp	USARTTXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
; definicje na potrzeby biblioteki obslugi USART
.equ	USART_Speed	= 47
;	USART BAUD Rate Generator fosc=7.3728 MHz , 9600 bps
.equ	USART_LED_Trace	= 0
.include "usartlib.inc"
;-----------------------------------------------------------------------------
.include "datesrv.inc"
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
.include "arithmetic.inc"
;-----------------------------------------------------------------------------
.include "strings.inc"
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ldi	param,USART_Speed		; USARTHardwareInit ( USART_Speed ) ;
	rcall	USARTHardwareInit		;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	rcall	USARTSoftwareInit		; USARTSoftwareInit ( ) ;
	ret					;END SoftwareInit ;
;-----------------------------------------------------------------------------
NewLine	:					;PROCEDURE NewLine ( ) ;
;********					;BEGIN (* NewLine *)
	ldi	param,Cr			; SendSerial ( Cr ) ;
	rcall	SendSerial			;
	ldi	param,Lf			; SendSerial ( Lf ) ;
	rcall	SendSerial			;
	ret					;END (* NewLine *) ;
;-----------------------------------------------------------------------------
DisplayDate :					;PROCEDURE DisplayDate ( ) ;
;************					;BEGIN (* DisplayDate *)
	lds	acc,DayNo			; Card := DayNo ;
	ldz	Card				;
	st	z+,acc				;
	ldi	acc,0				;
	st	z,acc				;
	ldz	Card				; WordToString ( Card , Number , 2 , ' ' ) ;
	ldx	Number				;
	ldi	param,2				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ldi	param,'.'			; SendSerial ( '.' ) ;
	rcall	SendSerial			;
	lds	acc,MonthNo			; Card := MonthNo ;
	ldz	Card				;
	st	z+,acc				;
	ldi	acc,0				;
	st	z,acc				;
	ldz	Card				; WordToString ( Card , Number , 2 , '0' ) ;
	ldx	Number				;
	ldi	param,2				;
	ldi	acc,'0'				;
	rcall	WordToString			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ldi	param,'.'			; SendSerial ( '.' ) ;
	rcall	SendSerial			;
	ldz	YearNo				; WordToString ( YearNo , Number , 4 , ' ' ) ;
	ldx	Number				;
	ldi	param,4				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ret					;END (* DisplayDate *) ;
;-----------------------------------------------------------------------------
DispDay :					;PROCEDURE DispDay ( ) ;
;********					;BEGIN (* DispDay *)
	ldz	Function3Text<<1		; SendSerialFlashTxt ( Function3Text ) ;
	rcall	SendSerialFlashTxt		;
	rcall	DisplayDate			; DisplayDate ( ) ;
	ldz	Function3AText<<1		; SendSerialFlashTxt ( Function3AText ) ;
	rcall	SendSerialFlashTxt		;
	ldx	YearNo				; CASE DayOfWeek ( YearNo , MonthNo , DayNo ) OF
	ld	zl,x+				;
	ld	zh,x				;
	lds	acc,MonthNo			;
	lds	param,DayNo			;
	rcall	DayOfWeek			;
	cpi	acc,Nie				;  Nie :
	brne	DisDy_0				;   BEGIN
	ldz	NiedzielaText<<1		;    SendSerialFlashTxt ( NiedzielaText ) ;
	rcall	SendSerialFlashTxt		;
	rjmp	DisDy_9				;   END (* CASE VARIANT *) ;
DisDy_0	:					;
	cpi	acc,Pon				;  Pon :
	brne	DisDy_1				;   BEGIN
	ldz	PoniedzText<<1			;    SendSerialFlashTxt ( PoniedzText ) ;
	rcall	SendSerialFlashTxt		;
	rjmp	DisDy_9				;   END (* CASE VARIANT *) ;
DisDy_1	:					;
	cpi	acc,Wto				;  Wto :
	brne	DisDy_2				;   BEGIN
	ldz	WtorekText<<1			;    SendSerialFlashTxt ( WtorekText ) ;
	rcall	SendSerialFlashTxt		;
	rjmp	DisDy_9				;   END (* CASE VARIANT *) ;
DisDy_2	:					;
	cpi	acc,Sro				;  Sro :
	brne	DisDy_3				;   BEGIN
	ldz	SrodaText<<1			;    SendSerialFlashTxt ( SrodaText ) ;
	rcall	SendSerialFlashTxt		;
	rjmp	DisDy_9				;   END (* CASE VARIANT *) ;
DisDy_3	:					;
	cpi	acc,Czw				;  Czw :
	brne	DisDy_4				;   BEGIN
	ldz	CzwartekText<<1			;    SendSerialFlashTxt ( CzwartekText ) ;
	rcall	SendSerialFlashTxt		;
	rjmp	DisDy_9				;   END (* CASE VARIANT *) ;
DisDy_4	:					;
	cpi	acc,Pia				;  Pia :
	brne	DisDy_5				;   BEGIN
	ldz	PiatekText<<1			;    SendSerialFlashTxt ( PiatekText ) ;
	rcall	SendSerialFlashTxt		;
	rjmp	DisDy_9				;   END (* CASE VARIANT *) ;
DisDy_5	:					;
	cpi	acc,Sob				;  Sob :
	brne	DisDy_6				;   BEGIN
	ldz	SobotaText<<1			;    SendSerialFlashTxt ( SobotaText ) ;
	rcall	SendSerialFlashTxt		;
	rjmp	DisDy_9				;   END (* CASE VARIANT *) ;
DisDy_6	:					;
DisDy_9 :					; END (* CASE *) ;
	rcall	NewLine				; NewLine ( ) ;
	ret					;END (* DispDay *) ;
;-----------------------------------------------------------------------------
ZmianaCzasuNaLetni :				;FUNCTION ZmianaCzasuNaLetni ( ) : BOOLEAN[flaga C] ;
;********************				;BEGIN (* ZmianaCzasuNaLetni *)
	lds	acc,MonthNo			; IF MonthNo = 3 THEN
	cpi	acc,3				;
	brne	ZmiaCNL_0			; BEGIN
	lds	acc,DayNo			;  IF DayNo < 25 THEN
	cpi	acc,25				;
	brcc	ZmiaCNL_1			;  BEGIN
	clc					;   ZmianaCzasuNaLetni := FALSE ;
	ret					;   RETURN ;
ZmiaCNL_1 :					;  END (* IF *) ;
	ldx	YearNo				;  IF DayOfWeek ( YearNo , MonthNo , DayNo ) = Nie THEN
	ld	zl,x+				;
	ld	zh,x				;
	lds	acc,MonthNo			;
	lds	param,DayNo			;
	rcall	DayOfWeek			;
	cpi	acc,Nie				;
	brne	ZmiaCNL_2			;  BEGIN
	sec					;   ZmianaCzasuNaLetni := TRUE ;
	ret					;   RETURN ;
ZmiaCNL_2 :					;  END (* IF *) ;
ZmiaCNL_0 :					; END (* IF *) ;
	clc					; ZmianaCzasuNaLetni := FALSE ;
	ret					;END (* ZmianaCzasuNaLetni *) ;
;-----------------------------------------------------------------------------
ZmianaCzasuNaZimowy :				;FUNCTION ZmianaCzasuNaZimowy ( ) : BOOLEAN[flaga C] ;
;********************				;BEGIN (* ZmianaCzasuNaZimowy *)
	lds	acc,MonthNo			; IF MonthNo = 10 THEN
	cpi	acc,10				;
	brne	ZmiaCNZ_0			; BEGIN
	lds	acc,DayNo			;  IF DayNo < 25 THEN
	cpi	acc,25				;
	brcc	ZmiaCNZ_1			;  BEGIN
	clc					;   ZmianaCzasuNaZimowy := FALSE ;
	ret					;   RETURN ;
ZmiaCNZ_1 :					;  END (* IF *) ;
	ldx	YearNo				;  IF DayOfWeek ( YearNo , MonthNo , DayNo ) = Nie THEN
	ld	zl,x+				;
	ld	zh,x				;
	lds	acc,MonthNo			;
	lds	param,DayNo			;
	rcall	DayOfWeek			;
	cpi	acc,Nie				;
	brne	ZmiaCNZ_2			;  BEGIN
	sec					;   ZmianaCzasuNaZimowy := TRUE ;
	ret					;   RETURN ;
ZmiaCNZ_2 :					;  END (* IF *) ;
ZmiaCNZ_0 :					; END (* IF *) ;
	clc					; ZmianaCzasuNaZimowy := FALSE ;
	ret					;END (* ZmianaCzasuNaZimowy *) ;
;-----------------------------------------------------------------------------
ResetProcessor	:				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
	ldz	Function1Text<<1		; SendSerialFlashTxt ( Function1Text ) ;
	rcall	SendSerialFlashTxt		;
	ldz	YearNo				; YearNo := 2006 ;
	ldi	acc,low(2006)			;
	st	z+,acc				;
	ldi	acc,high(2006)			;
	st	z,acc				;
	ldi	acc,3				; MonthNo := 3 ;
	sts	MonthNo,acc			;
	ldi	acc,1				; DayNo := 1 ;
	sts	DayNo,acc			;
Main_0	:					; LOOP
	rcall	ZmianaCzasuNaLetni		;  IF ZmianaCzasuNaLetni ( ) THEN
	brcc	Main_1				;  BEGIN
	rcall	DisplayDate			;   DisplayDate ( ) ;
	rcall	NewLine				;   NewLine ( ) ;
	rjmp	Main_2				;   EXIT ;
Main_1	:					;  END (* IF *) ;
	ldz	DayNo				;  DayNo := DayNo + 1 ;
	ld	acc,z				;
	inc	acc				;
	st	z,acc				;
	rjmp	Main_0				; END (* LOOP *) ;
Main_2	:					;
	ldz	Function2Text<<1		; SendSerialFlashTxt ( Function2Text ) ;
	rcall	SendSerialFlashTxt		;
	ldi	acc,10				; MonthNo := 10 ;
	sts	MonthNo,acc			;
	ldi	acc,1				; DayNo := 1 ;
	sts	DayNo,acc			;
Main_4	:					; LOOP
	rcall	ZmianaCzasuNaZimowy		;  IF ZmianaCzasuNaZimowy ( ) THEN
	brcc	Main_5				;  BEGIN
	rcall	DisplayDate			;   DisplayDate ( ) ;
	rcall	NewLine				;   NewLine ( ) ;
	rjmp	Main_6				;   EXIT ;
Main_5	:					;  END (* IF *) ;
	ldz	DayNo				;  DayNo := DayNo + 1 ;
	ld	acc,z				;
	inc	acc				;
	st	z,acc				;
	rjmp	Main_4				; END (* LOOP *) ;
Main_6	:					;
	ldz	YearNo				; YearNo := 1994 ;
	ldi	acc,low(1994)			;
	st	z+,acc				;
	ldi	acc,high(1994)			;
	st	z,acc				;
	ldi	acc,5				; MonthNo := 5 ;
	sts	MonthNo,acc			;
	ldi	acc,23				; DayNo := 23 ;
	sts	DayNo,acc			;
	rcall	DispDay				; DispDay ( ) ;
	ldz	YearNo				; YearNo := 2007 ;
	ldi	acc,low(2007)			;
	st	z+,acc				;
	ldi	acc,high(2007)			;
	st	z,acc				;
	ldi	acc,1				; MonthNo := 1 ;
	sts	MonthNo,acc			;
	ldi	acc,1				; DayNo := 1 ;
	sts	DayNo,acc			;
	rcall	DispDay				; DispDay ( ) ;
	ldz	Function4Text<<1		; SendSerialFlashTxt ( Function4Text ) ;
	rcall	SendSerialFlashTxt		;
	ldz	YearNo				; YearNo := 1997 ;
	ldi	acc,low(1997)			;
	st	z+,acc				;
	ldi	acc,high(1997)			;
	st	z,acc				;
	ldi	acc,1				; MonthNo := 1 ;
	sts	MonthNo,acc			;
	ldi	acc,15				; DayNo := 15 ;
	sts	DayNo,acc			;
	rcall	DisplayDate			; DisplayDate ( ) ;
	ldx	YearNo				; Date := ConvDateToNumber ( YearNo , MonthNo , DayNo ) ;
	ld	zl,x+				;
	ld	zh,x				;
	lds	acc,MonthNo			;
	lds	param,DayNo			;
	rcall	ConvDateToNumber		;
	ldz	Date				;
	st	z+,xl				;
	st	z,xh				;
	ldz	Function4AText<<1		; SendSerialFlashTxt ( Function4AText ) ;
	rcall	SendSerialFlashTxt		;
	ldz	YearNo				; YearNo := 2003 ;
	ldi	acc,low(2003)			;
	st	z+,acc				;
	ldi	acc,high(2003)			;
	st	z,acc				;
	ldi	acc,9				; MonthNo := 9 ;
	sts	MonthNo,acc			;
	ldi	acc,23				; DayNo := 23 ;
	sts	DayNo,acc			;
	rcall	DisplayDate			; DisplayDate ( ) ;
	ldx	YearNo				; Card := ConvDateToNumber ( YearNo , MonthNo , DayNo ) - Date ;
	ld	zl,x+				;
	ld	zh,x				;
	lds	acc,MonthNo			;
	lds	param,DayNo			;
	rcall	ConvDateToNumber		;
	ldz	Date				;
	ld	acc,z+				;
	ld	acc2,z				;
	sub	xl,acc				;
	sbc	xh,acc2				;
	ldz	Card				;
	st	z+,xl				;
	st	z,xh				;
	ldz	Function4BText<<1		; SendSerialFlashTxt ( Function4BText ) ;
	rcall	SendSerialFlashTxt		;
	ldz	Card				; WordToString ( Card , Number , 4 , ' ' ) ;
	ldx	Number				;
	ldi	param,4				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; SendSerialRAMTxt ( Number ) ;
	rcall	SendSerialRAMTxt		;
	ldz	Function4CText<<1		; SendSerialFlashTxt ( Function4CText ) ;
	rcall	SendSerialFlashTxt		;
Main_7	:					; HALT ( ) ;
	rjmp	Main_7				;	
						;END (* Main *).
;-----------------------------------------------------------------------------
Function1Text:	.db	"Zmiana czasu na letni w :",0
Function2Text:	.db	"Zmiana czasu na zimowy w :",0,0
Function3Text:	.db	"Dzien: ",0
Function3AText:	.db	" to: ",0
Function4Text:	.db	"Od dnia: ",0
Function4AText:	.db	" do dnia: ",0,0
Function4BText:	.db	" minelo: ",0
Function4CText:	.db	" dni.",Cr,Lf,Cr,Lf,0
NiedzielaText:	.db	"niedziela",0
PoniedzText:	.db	"poniedzialek",0,0
WtorekText:	.db	"wtorek",0,0
SrodaText:	.db	"sroda",0
CzwartekText:	.db	"czwartek",0,0
PiatekText:	.db	"piatek",0,0
SobotaText:	.db	"sobota",0,0
.exit
