;
;*******************************************************
;*******************************************************
;**                                                   **
;**      Obsluga pamieci nieulotnej EEPROM            **
;**                 wariant dwa                       **
;**                                                   **
;**              Andrzej Pawluczuk                    **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
;			= r0 [reserved]
.def DivLoopCounter	= r1
.def Argument1LL	= r2
.def Argument1LH	= r3
.def Argument1HL	= r4
.def Argument1HH	= r5
.def Argument2LL	= r6
.def Argument2LH	= r7
.def Argument2HL	= r8
.def Argument2HH	= r9
.def WorkRegLL		= r10
.def WorkRegLH		= r11
.def WorkRegHL		= r12
.def WorkRegHH		= r13
.def acc		= r16
.def acc2		= r17
.def param		= r18
.def ResultLL		= r22
.def ResultLH		= r23
.def ResultHL		= r24
.def ResultHH		= r25
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b00000000
;					  ||||||||
;					  |||||||+-- input : KEY
;					  ||||||+--- input : KEY
;					  |||||+---- input : KEY
;					  ||||+----- input : KEY
;					  |||+------ input : KEY
;					  ||+------- input : KEY
;					  |+-------- input : KEY
;					  +--------- input : KEY
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : LDCD data bus
;					  ||||||+--- output : LDCD data bus
;					  |||||+---- output : LDCD data bus
;					  ||||+----- output : LDCD data bus
;					  |||+------ output : LCD RS
;					  ||+------- output : LCD R/W
;					  |+-------- output : LCD E
;					  +--------- output :
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output :
;					  ||||||+--- output :
;					  |||||+---- output :
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b00000001
;					  ||||||||
;					  |||||||+-- output:
;					  ||||||+--- input :
;					  |||||+---- input :
;					  ||||+----- input :
;					  |||+------ input :
;					  ||+------- input :
;					  |+-------- input :
;					  +--------- input :
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
.equ	Key0Pin		= 7
.equ	Key1Pin		= 6
.equ	Key2Pin		= 5
.equ	Key3Pin		= 4
.equ	Key4Pin		= 3
.equ	Key5Pin		= 2
.equ	Key6Pin		= 1
.equ	Key7Pin		= 0
;-----------------------------------------------------------------------------
.equ	Key0Code	= '0'
.equ	Key1Code	= '1'
.equ	Key2Code	= '2'
.equ	Key3Code	= '3'
.equ	Key4Code	= '4'
.equ	Key5Code	= '5'
.equ	Key6Code	= '6'
.equ	Key7Code	= '7'
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;-----------------------------------------------------------------------------
;					;TYPE
;					;CONST
.equ	OptionLimit	= 3		; OptionLimit = 3 ;
;					;
;					;VAR
Number :	.byte	8		; Number : ARRAY [ 0 .. 7 ] OF CHAR ;
DispMenuReq :	.byte	1		; DispMenuReq : BOOLEAN ;
Option :	.byte	1		; Option : BYTE ;
ParamV :	.byte	2		; ParamV : WORD ;
EEPROMTextData:	.byte	8		; EEPROMTextData : ARRAY [ 0 .. 7 ] OF CHAR ;
	.eseg
					;CONST
.equ	EEPROMDataSize	= 14		; EEPROMDataSize = 14 ;
					;VAR
EEPROMBl_Param1 :	.byte 1		; EEPROMBl_Param1 : CHAR ;
EEPROMBl_Param2 :	.byte 2		; EEPROMBl_Param2 : WORD ;
EEPROMBl_Param3 :	.byte 8		; EEPROMBl_Param3 : ARRAY [ 0 .. 7 ] OF CHAR ;
EEPROMBl_Param4 :	.byte 1		; EEPROMBl_Param4 : BYTE ;
EEPROMBl_CRC :		.byte 2		; EEPROMBl_CRC : WORD ;
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;-----------------------------------------------------------------------------
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;UART Receive Complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;UART Data Register Empty Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;UART Transmit Complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
.equ	lcd_Interf4b	= 1
.equ	lcd_E_port	= PORTB
.equ	lcd_E_pin	= 6
.equ	lcd_RW_port	= PORTB
.equ	lcd_RW_pin	= 5
.equ	lcd_RS_port	= PORTB
.equ	lcd_RS_pin	= 4
.equ	lcdD0		= 0
.equ	lcdD1		= 1
.equ	lcdD2		= 2
.equ	lcdD3		= 3
.equ	lcd_Data_port	= PORTB
.include "lcdlib.inc"
;-----------------------------------------------------------------------------
.equ	KeyPort		= PORTA
.equ	KeyPDirection	= KeyPort - 1
.equ	KeyPInput	= KeyPort - 2
.include "keybsrv.inc"
;-----------------------------------------------------------------------------
.include "arithmetic.inc"
;-----------------------------------------------------------------------------
.include "strings.inc"
;-----------------------------------------------------------------------------
.include "strlib.inc"
;-----------------------------------------------------------------------------
.include "crc16lib.inc"
;-----------------------------------------------------------------------------
TC0OvfInterrupt	:				;PROCEDURE TC0OvfInterrupt ( ) ;
;****************				;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	rcall	KeybService			; KeybService ( ) ;
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ldi	acc,0				; KeyPort := <input> + <pullup>;
	out	KeyPDirection,acc		;
	ldi	acc,0xFF			;
	out	KeyPort,acc			;
	rcall	InitLCDEnvir			; InitLCDEnvir ( ) ;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	rcall	KeybSoftInit			; KeybSoftInit ( ) ;
	ldi	acc,1				; DispMenuReq := TRUE ;
	sts	DispMenuReq,acc			;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
DecodeKey :					;PROCEDURE DecodeKey ( ) ;
;**********					;VAR
;						; Kbd [acc2] : BYTE ;
;						; Temp [acc] : BYTE ;
						;BEGIN (* DecodeKey *)
	in	acc2,KeyPInput			; Kbd := KeyPInput ;
	ldi	acc,0				; Temp := 0 ;
	sbrs	acc2,Key0Pin			; IF Kbd[Key0Pin] THEN
	ldi	acc,Key0Code			;  Temp := Key0Code ;
	sbrs	acc2,Key1Pin			; IF Kbd[Key1Pin] THEN
	ldi	acc,Key1Code			;  Temp := Key1Code ;
	sbrs	acc2,Key2Pin			; IF Kbd[Key2Pin] THEN
	ldi	acc,Key2Code			;  Temp := Key2Code ;
	sbrs	acc2,Key3Pin			; IF Kbd[Key3Pin] THEN
	ldi	acc,Key3Code			;  Temp := Key3Code ;
	sbrs	acc2,Key4Pin			; IF Kbd[Key4Pin] THEN
	ldi	acc,Key4Code			;  Temp := Key4Code ;
	sbrs	acc2,Key5Pin			; IF Kbd[Key5Pin] THEN
	ldi	acc,Key5Code			;  Temp := Key5Code ;
	sbrs	acc2,Key6Pin			; IF Kbd[Key6Pin] THEN
	ldi	acc,Key6Code			;  Temp := Key6Code ;
	sbrs	acc2,Key7Pin			; IF Kbd[Key7Pin] THEN
	ldi	acc,Key7Code			;  Temp := Key7Code ;
	rcall	StoreKey			; StoreKey ( Temp ) ;
	ret					;END (* DecodeKey *) ;
;-----------------------------------------------------------------------------
AnyKeyActive :					;FUNCTION AnyKeyActive ( ) : BOOLEAN [C] ;
;*************					;BEGIN (* AnyKeyActive *)
	in	acc,KeyPInput			; IF KeyPort = 0xFF THEN
	cpi	acc,0xFF			;
	brne	AnKeAct_0			; BEGIN
	clc					;  AnyKeyActive := FALSE ;
	ret					;  RETURN ;
AnKeAct_0 :					; END (* IF *) ;
	sec					; AnyKeyActive := TRUE ;
	ret					;END (* AnyKeyActive *) ;
;-----------------------------------------------------------------------------
EEPROMRead :					;FUNCTION EEPROMRead ( Offset[param] : BYTE ) : byte[acc] ;
;***********					;BEGIN (* EEPROMRead *)
EERd_0	:					; WHILE EECR . EEWE = 1 DO
	sbic	EECR,EEWE			; BEGIN
	rjmp	EERd_0				; END (* WHILE *) ;
	ldi	acc,0				; EEAR := Offset ;
	out	EEARH,acc			;
	out	EEARL,param			;
	sbi	EECR,EERE			; EECR . EERE := 1 ;
	in	acc,EEDR			; EEPROMRead := EEDR ;
	ret					;END (* EEPROMRead *) ;
;-----------------------------------------------------------------------------
EEPROMWrite :					;PROCEDURE EEPROMWrite ( Offset[acc2] : BYTE ;
;************					;			 Data[param] : BYTE ) ;
;						;BEGIN (* EEPROMWrite *)
EEWr_0	:					; WHILE EECR . EEWE = 1 DO
	sbic	EECR,EEWE			; BEGIN
	rjmp	EEWr_0				; END (* WHILE *) ;
	ldi	acc,0				; EEAR := Offset ;
	out	EEARH,acc			;
	out	EEARL,acc2			;
	out	EEDR,param			; EEDR := Data ;
	cli					; DI ( ) ;
	sbi	EECR,EEMWE			; EECR . EEMWE := 1 ;
	sbi	EECR,EEWE			; EECR . EEWE := 1 ;
	sei					; EI ( ) ;
	ret					;END (* EEPROMWrite *) ;
;-----------------------------------------------------------------------------
UpdateCRC :					;PROCEDURE UpdateCRC ( ) ;
;**********					;VAR
;						; CRCData[x] : WORD ;
;						; Loop[acc2]:BYTE ;
;						;BEGIN (* UpdateCRC *)
	rcall	StartCRC			; StartCRC ( ) ;
	clr	acc2				; FOR Loop := 0 TO EEPROMDataSize - 3 DO
UpdCRC_0 :					; BEGIN
	push	acc2				;
	mov	param,acc2			;  CRCStep ( EEPROMRead ( Loop ) ) ;
	rcall	EEPROMRead			;
	mov	param,acc			;
	rcall	CRCStep				;
	pop	acc2				;
	inc	acc2				;
	cpi	acc2,EEPROMDataSize-2		;
	brne	UpdCRC_0			; END (* FOR *) ;
	rcall	GetCRCSum			; CRCData := GetCRCSum ( ) ;
	push	xh				;
	ldi	acc2,low(EEPROMBl_CRC)		; EEPROMWrite ( ADR ( EEPROMBl_CRC ) , CRCData MOD 256 ) ;
	mov	param,xl			;
	rcall	EEPROMWrite			;
	ldi	acc2,low(EEPROMBl_CRC+1)	; EEPROMWrite ( ADR ( EEPROMBl_CRC + 1 ) , CRCData DIV 256 ) ;
	pop	param				;
	rcall	EEPROMWrite			;
	ret					;END (* UpdateCRC *) ;
;-----------------------------------------------------------------------------
GetParam1 :					;FUNCTION GetParam1 ( ) : BYTE[acc] ;
;**********					;BEGIN (* GetParam1 *)
	ldi	param,low(EEPROMBl_Param1)	; GetParam1 := EEPROMRead ( ADR ( EEPROMBl_Param1 ) ) ;
	rcall	EEPROMRead			;
	ret					;END (* GetParam1 *) ;
;-----------------------------------------------------------------------------
GetParam2 :					;FUNCTION GetParam2 ( ) : WORD[x] ;
;**********					;VAR
;						; EEDataL[xl] : BYTE ;
;						; EEDataH[xh] : BYTE ;
;						;BEGIN (* GetParam2 *)
	ldi	param,low(EEPROMBl_Param2)	; EEDataL := EEPROMRead ( ADR ( EEPROMBl_Param2 ) ) ;
	rcall	EEPROMRead			;
	push	acc				;
	ldi	param,low(EEPROMBl_Param2+1)	; EEDataH := EEPROMRead ( ADR ( EEPROMBl_Param2 + 1 ) ) ;
	rcall	EEPROMRead			;
	mov	xh,acc				; GetParam2 := EEDataH << 8 + EEDataL ;
	pop	xl				;
	ret					;END (* GetParam2 *) ;
;-----------------------------------------------------------------------------
GetParam3 :					;PROCEDURE GetParam3 ( ) ;
;**********					;VAR
;						; Loop[acc2] : BYTE ;
;						;BEGIN (* GetParam3 *)
	ldi	acc2,0				; FOR Loop = 0 TO 7 DO
	ldz	EEPROMTextData			;
GePa3_0	:					; BEGIN
	ldi	param,low(EEPROMBl_Param3)	;  EEPROMTextData [ Loop ] := EEPROMRead ( ADR ( EEPROMBl_Param3 ) + Loop ) ;
	add	param,acc2			;
	rcall	EEPROMRead			;
	st	z+,acc				;
	cpi	acc2,7				;
	breq	GePa3_1				;
	inc	acc2				;
	rjmp	GePa3_0				; END (* FOR *) ;
GePa3_1	:					;
	ret					;END (* GetParam3 *) ;
;-----------------------------------------------------------------------------
GetParam4 :					;FUNCTION GetParam4 ( ) : BYTE[acc] ;
;**********					;BEGIN (* GetParam4 *)
	ldi	param,low(EEPROMBl_Param4)	; GetParam4 := EEPROMRead ( ADR ( EEPROMBl_Param4 ) ) ;
	rcall	EEPROMRead			;
	ret					;END (* GetParam4 *) ;
;-----------------------------------------------------------------------------
SaveParam1 :					;PROCEDURE SaveParam1 ( Data[param] : BYTE ) ;
;**********					;BEGIN (* SaveParam1 *)
	ldi	acc2,low(EEPROMBl_Param1)	; EEPROMWrite ( ADR ( EEPROMBl_Param1 ) , Data ) ;
	rcall	EEPROMWrite			;
	rcall	UpdateCRC			; UpdateCRC ( ) ;
	ret					;END (* SaveParam1 *) ;
;-----------------------------------------------------------------------------
SaveParam2 :					;PROCEDURE SaveParam2 ( Data[z] : WORD ) ;
;**********					;BEGIN (* SaveParam2 *)
	ldi	acc2,low(EEPROMBl_Param2)	; EEPROMWrite ( ADR ( EEPROMBl_Param2 ) , Data MOD 256 ) ;
	mov	param,zl			;
	rcall	EEPROMWrite			;
	ldi	acc2,low(EEPROMBl_Param2+1)	; EEPROMWrite ( ADR ( EEPROMBl_Param2 + 1 ) , Data DIV 256 ) ;
	mov	param,zh			;
	rcall	EEPROMWrite			;
	rcall	UpdateCRC			; UpdateCRC ( ) ;
	ret					;END (* SaveParam2 *) ;
;-----------------------------------------------------------------------------
SaveParam3 :					;PROCEDURE SaveParam3 ( ) ;
;**********					;VAR
;						; Loop[acc2] : BYTE ;
;						;BEGIN (* SaveParam3 *)
	ldi	acc2,0				; FOR Loop = 0 TO 7 DO
	ldz	EEPROMTextData			;
SvPa3_0	:					; BEGIN
	push	acc2				;  EEPROMWrite ( ADR ( EEPROMBl_Param3 ) + Loop ) , EEPROMTextData [ Loop ] ) ;
	ldi	acc,low(EEPROMBl_Param3)	;
	add	acc2,acc			;
	ld	param,z+			;
	rcall	EEPROMWrite			;
	pop	acc2				;
	cpi	acc2,7				;
	breq	SvPa3_1				;
	inc	acc2				;
	rjmp	SvPa3_0				; END (* FOR *) ;
SvPa3_1	:					;
	rcall	UpdateCRC			; UpdateCRC ( ) ;
	ret					;END (* SaveParam3 *) ;
;-----------------------------------------------------------------------------
SaveParam4 :					;PROCEDURE SaveParam4 ( Data[param] : BYTE ) ;
;**********					;BEGIN (* SaveParam4 *)
	ldi	acc2,low(EEPROMBl_Param4)	; EEPROMWrite ( ADR ( EEPROMBl_Param4 ) , Data ) ;
	rcall	EEPROMWrite			;
	rcall	UpdateCRC			; UpdateCRC ( ) ;
	ret					;END (* SaveParam4 *) ;
;-----------------------------------------------------------------------------
LoadParam :					;FUNCTION LoadParam ( ) : BOOLEAN ;
;**********					;VAR
;						; Loop[acc2] : BYTE ;
;						; EEData[acc] : BYTE ;
;						;BEGIN (* LoadParam *)
	rcall	StartCRC			; StartCRC ( ) ;
	clr	acc2				; FOR Loop := 0 TO EEPROMDataSize - 1 DO
LoPa_0	:					; BEGIN
	mov	param,acc2			;  CRCStep ( EEPROMRead ( Loop ) ) ;
	push	acc2				;
	rcall	EEPROMRead			;
	mov	param,acc			;
	rcall	CRCStep				;
	pop	acc2				;
	inc	acc2				;
	cpi	acc2,EEPROMDataSize		;
	brne	LoPa_0				; END (* FOR *) ;
	rcall	GetCRCSum			; IF GetCRCSum ( ) = 0 THEN
	mov	acc,XL				;
	or	acc,XH				;
	brne	LoPa_1				; BEGIN
	sec					;  LoadParam := TRUE ;
	ret					;
LoPa_1	:					; ELSE
	clc					;  LoadParam := FALSE ;
	ret					;
						; END (* IF ... ELSE *) ;
						;END (* LoadParam *) ;
;-----------------------------------------------------------------------------
DefaultParam :					;PROCEDURE DefaultParam ( ) ;
;*************					;BEGIN (* DefaultParam *)
	ldi	param,'A'			; SaveParam1 ( 'A' ) ;
	rcall	SaveParam1			;
	ldz	2005				; SaveParam2 ( 2005 ) ;
	rcall	SaveParam2			;
	ldx	EEPROMTextData			; StrCopyFlash ( EEPROMTextData , AVRTxt ) ;
	ldz	AVRTxt<<1			;
	rcall	StrCopyFlash			;
	rcall	SaveParam3			; SaveParam3 ( ) ;
	ldi	param,0x80			; SaveParam4 ( 0x80 ) ;
	rcall	SaveParam4			;
	ret					;END DefaultParam ;
;-----------------------------------------------------------------------------
DisplayMessage :				;PROCEDURE DisplayMessage ( MessTxt[z] : POINTER TO ARRAY OF CHAR ) ;
;*************					;BEGIN (* DisplayMessage *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	rcall	WriteTextLCDFlash		; WriteTextLCDFlash ( MessTxt ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	ret					;END (* DisplayMessage *) ;
;-----------------------------------------------------------------------------
AnyKey	:					;PROCEDURE AnyKey ( ) ;
;********					;BEGIN (* AnyKey *)
AnyK_0	:					; WHILE NOT KeyPressed ( ) DO
	rcall	KeyPressed			; BEGIN
	brcc	AnyK_0				; END (* WHILE *) ;
	rcall	ReadKey				; ? := ReadKey ( ) ;
	ret					;END (* AnyKey *) ;
;-----------------------------------------------------------------------------
DisplayParam2 :					;PROCEDURE DisplayParam2 ( ) ;
;**************					;BEGIN (* DisplayParam2 *)
	rcall	GetParam2			; ParamV := GetParam2 ( ) ;
	ldz	ParamV				;
	st	z+,xl				;
	st	z,xh				;
	ldz	ParamV				; WordToString ( ADR ( ParamV ) , Number , 5 , ' ' ) ;
	ldx	Number				;
	ldi	param,5				;
	ldi	acc,' '				;
	rcall	WordToString			;
	ldz	Number				; WriteTextLCD ( Number ) ;
	rcall	WriteTextLCD			;
	ret					;END (* DisplayParam2 *) ;
;-----------------------------------------------------------------------------
DisplayParam4 :					;PROCEDURE DisplayParam4 ( ) ;
;**************					;VAR
;						; HexStr[x] : ARRAY [ 0 .. 1 ] OF CHAR ;
;						;BEGIN (* DisplayParam4 *)
	rcall	GetParam4			; HexStr := ByteToHex ( GetParam4 ( ) ) ;
	mov	param,acc			;
	rcall	ByteToHex			;
	push	xl				;
	mov	param,xh			; WriteChLCD ( HexStr [ 0 ] ) ;
	rcall	WriteChLCD			;
	pop	param				; WriteChLCD ( HexStr [ 1 ] ) ;
	rcall	WriteChLCD			;
	ret					;END (* DisplayParam4 *) ;
;-----------------------------------------------------------------------------
Option1Service :				;PROCEDURE Option1Service ( ) ;
;***************				;VAR
;						; Data[acc] : BYTE ;
;						;BEGIN (* Option1Service *)
	rcall	GetParam1			; Data := GetParam1 ( ) ;
	ldi	acc2,0x20			; Data := EXOR ( Data , 0x20 ) ;
	eor	acc,acc2			;
	mov	param,acc			; SaveParam1 ( Data ) ;
	rcall	SaveParam1			;
	ret					;END (* Option1Service *) ;
;-----------------------------------------------------------------------------
Option2Service :				;PROCEDURE Option2Service ( ) ;
;***************				;VAR
;						; Data[x] : WORD ;
;						;BEGIN (* Option2Service *)
Op2Srv_0 :					; LOOP
	rcall	ClrScrLCD			;  ClrScrLCD ( ) ;
	ldz	Opt2Prompt<<1			;  WriteTextLCDFlash ( Opt2Prompt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	DisplayParam2			;  DisplayParam2 ( ) ;
Op2Srv_1 :					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	Op2Srv_1			;
	rcall	ReadKey				;  CASE ReadKey ( ) OF
	cpi	acc,Key4Code			;  | Key4Code  :
	brne	Op2Srv_4			;   BEGIN
	rcall	GetParam2			;    Data := GetParam2 ( ) ;
	ldi	acc,1				;    Data := Data + 1 ;
	add	xl,acc				;
	ldi	acc,0				;
	adc	xh,acc				;
	mov	zl,xl				;    SaveParam2 ( Data ) ;
	mov	zh,xh				;
	rcall	SaveParam2			;
	rjmp	Op2Srv_7			;   END (* CASE VARIANT *) ;
Op2Srv_4 :					;
	cpi	acc,Key5Code			;  | Key5Code  :
	brne	Op2Srv_5			;   BEGIN
	rcall	GetParam2			;    Data := GetParam2 ( ) ;
	ldi	acc,0xFF			;    Data := Data - 1 ;
	add	xl,acc				;
	adc	xh,acc				;
	mov	zl,xl				;    SaveParam2 ( Data ) ;
	mov	zh,xh				;
	rcall	SaveParam2			;
	rjmp	Op2Srv_7			;   END (* CASE VARIANT *) ;
Op2Srv_5 :					;
	cpi	acc,Key6Code			;  | Key6Code  :
	brne	Op2Srv_6			;   BEGIN
	ret					;    RETURN ;
Op2Srv_6 :					;   END (* CASE VARIANT *) ;
Op2Srv_7 :					;  END (* CASE *) ;
	rjmp	Op2Srv_0			; END (* LOOP *) ;
	ret					;END (* Option2Service *) ;
;-----------------------------------------------------------------------------
Option3Service :				;PROCEDURE Option3Service ( ) ;
;***************				;BEGIN (* Option3Service *)
	ldi	acc,0				; Option := 0 ;
	sts	Option,acc			;
Op3Srv_0 :					; LOOP
	rcall	ClrScrLCD			;  ClrScrLCD ( ) ;
	ldi	param,'['			;  WriteChLCD ( '[' ) ;
	rcall	WriteChLCD			;
	lds	param,Option			;  WriteChLCD ( Option + 1 + '0' ) ;
	ldi	acc,'0'+1			;
	add	param,acc			;
	rcall	WriteChLCD			;
	ldi	param,']'			;  WriteChLCD ( ']' ) ;
	rcall	WriteChLCD			;
	ldi	param,':'			;  WriteChLCD ( ':' ) ;
	rcall	WriteChLCD			;
	ldz	ValidPar3Table<<1		;  WriteTextLCDFlash ( ValidPar3Table [ Option ] ) ;
	lds	acc,Option			;
	lsl	acc				;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;
	push	r0				;
	adiw	zl,1				;
	lpm					;
	mov	zh,r0				;
	pop	zl				;
	lsl	zl				;
	rol	zh				;
	rcall	WriteTextLCDFlash		;
Op3Srv_1 :					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	Op3Srv_1			;
	rcall	ReadKey				;  CASE ReadKey ( ) OF
	cpi	acc,Key4Code			;  | Key4Code  :
	brne	Op3Srv_4			;   BEGIN
	ldz	Option				;    IF Option < 7 THEN
	ld	acc,z				;
	cpi	acc,7				;
	brcc	Op3Srv_41			;    BEGIN
	inc	acc				;     Option := Option + 1 ;
	st	z,acc				;
Op3Srv_41 :					;    END (* IF *) ;
	rjmp	Op3Srv_7			;   END (* CASE VARIANT *) ;
Op3Srv_4 :					;
	cpi	acc,Key5Code			;  | Key5Code  :
	brne	Op3Srv_5			;   BEGIN
	ldz	Option				;    IF Option <> 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	breq	Op3Srv_51			;    BEGIN
	dec	acc				;     Option := Option - 1 ;
	st	z,acc				;
Op3Srv_51 :					;    END (* IF *) ;
	rjmp	Op3Srv_7			;   END (* CASE VARIANT *) ;
Op3Srv_5 :					;
	cpi	acc,Key6Code			;  | Key6Code  :
	brne	Op3Srv_6			;   BEGIN
	ldz	ValidPar3Table<<1		;    StrCopyFlash ( EEPROMTextData , ValidPar3Table [ Option ] ) ;
	lds	acc,Option			;
	lsl	acc				;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;
	push	r0				;
	adiw	zl,1				;
	lpm					;
	mov	zh,r0				;
	pop	zl				;
	lsl	zl				;
	rol	zh				;
	ldx	EEPROMTextData			;
	rcall	StrCopyFlash			;
	rcall	SaveParam3			;    SaveParam3 ( ) ;
	ret					;    RETURN ;
Op3Srv_6 :					;   END (* CASE VARIANT *) ;
Op3Srv_7 :					;  END (* CASE *) ;
	rjmp	Op3Srv_0			; END (* LOOP *) ;
	ret					;END (* Option3Service *) ;
;-----------------------------------------------------------------------------
Option4Service :				;PROCEDURE Option4Service ( ) ;
;***************				;VAR
;						; Data[acc] : BYTE ;
;						;BEGIN (* Option4Service *)
Op4Srv_0 :					; LOOP
	rcall	ClrScrLCD			;  ClrScrLCD ( ) ;
	ldz	Opt4Prompt<<1			;  WriteTextLCDFlash ( Opt4Prompt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	DisplayParam4			;  DisplayParam4 ( ) ;
Op4Srv_1 :					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	Op4Srv_1			;
	rcall	ReadKey				;  CASE ReadKey ( ) OF
	cpi	acc,Key4Code			;  | Key4Code  :
	brne	Op4Srv_4			;   BEGIN
	rcall	GetParam4			;    Data := GetParam4 ( ) ;
	cpi	acc,0x80			;    IF Data <> 0x80 THEN
	breq	Op4Srv_41			;    BEGIN
	lsl	acc				;     Data := Data << 1 ;
	mov	param,acc			;     SaveParam4 ( Data ) ;
	rcall	SaveParam4			;
	st	z,acc				;
Op4Srv_41 :					;    END (* IF *) ;
	rjmp	Op4Srv_7			;   END (* CASE VARIANT *) ;
Op4Srv_4 :					;
	cpi	acc,Key5Code			;  | Key5Code  :
	brne	Op4Srv_5			;   BEGIN
	rcall	GetParam4			;    Data := GetParam4 ( ) ;
	cpi	acc,0x01			;    IF Data <> 0x01 THEN
	breq	Op4Srv_51			;    BEGIN
	lsr	acc				;     Data := Data >> 1 ;
	mov	param,acc			;     SaveParam4 ( Data ) ;
	rcall	SaveParam4			;
Op4Srv_51 :					;    END (* IF *) ;
	rjmp	Op4Srv_7			;   END (* CASE VARIANT *) ;
Op4Srv_5 :					;
	cpi	acc,Key6Code			;  | Key6Code  :
	brne	Op4Srv_6			;   BEGIN
	ret					;    RETURN ;
Op4Srv_6 :					;   END (* CASE VARIANT *) ;
Op4Srv_7 :					;  END (* CASE *) ;
	rjmp	Op4Srv_0			; END (* LOOP *) ;
	ret					;END (* Option4Service *) ;
;-----------------------------------------------------------------------------
DisplayParam :					;PROCEDURE DisplayParam ( ) ;
;*************					;BEGIN (* DisplayParam *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	Param1PromptTxt<<1		; WriteTextLCDFlash ( Param1PromptTxt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	GetParam1			; WriteChLCD ( GetParam1 ( ) ) ;
	mov	param,acc			;
	rcall	WriteChLCD			;
	rcall	AnyKey				; AnyKey ( ) ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	Param2PromptTxt<<1		; WriteTextLCDFlash ( Param2PromptTxt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	DisplayParam2			; DisplayParam2 ( ) ;
	rcall	AnyKey				; AnyKey ( ) ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	Param3PromptTxt<<1		; WriteTextLCDFlash ( Param3PromptTxt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	GetParam3			; GetParam3 ( ) ;
	ldz	EEPROMTextData			; WriteTextLCD ( EEPROMTextData ) ;
	rcall	WriteTextLCD			;
	rcall	AnyKey				; AnyKey ( ) ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	Param4PromptTxt<<1		; WriteTextLCDFlash ( Param4PromptTxt ) ;
	rcall	WriteTextLCDFlash		;
	rcall	DisplayParam4			; DisplayParam4 ( ) ;
	rcall	AnyKey				; AnyKey ( ) ;
	ret					;END (* DisplayParam *) ;
;-----------------------------------------------------------------------------
ModiParamSrv :					;PROCEDURE ModiParamSrv ( ) ;
;*************					;BEGIN (* ModiParamSrv *)
	ldi	acc,0				; Option := 0 ;
	sts	Option,acc			;
MoPar_0 :					; LOOP
	rcall	ClrScrLCD			;  ClrScrLCD ( ) ;
	ldz	OptionTxt<<1			;  WriteTextLCDFlash ( OptionTxt [ Option ] ) ;
	lds	acc,Option			;
	lsl	acc				;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;
	push	r0				;
	adiw	zl,1				;
	lpm					;
	mov	zh,r0				;
	pop	zl				;
	lsl	zl				;
	rol	zh				;
	rcall	WriteTextLCDFlash		;
MoPar_1	:					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	MoPar_1				;
	rcall	ReadKey				;  CASE ReadKey ( ) OF
	cpi	acc,Key4Code			;  | Key4Code  :
	brne	MoPar_4				;   BEGIN
	ldz	Option				;    IF Option < OptionLimit THEN
	ld	acc,z				;
	cpi	acc,OptionLimit			;
	brcc	MoPar_41			;    BEGIN
	inc	acc				;     Option := Option + 1 ;
	st	z,acc				;
MoPar_41 :					;    END (* IF *) ;
	rjmp	MoPar_8				;   END (* CASE VARIANT *) ;
MoPar_4	:					;
	cpi	acc,Key5Code			;  | Key5Code  :
	brne	MoPar_5				;   BEGIN
	ldz	Option				;    IF Option <> 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	breq	MoPar_51			;    BEGIN
	dec	acc				;     Option := Option - 1 ;
	st	z,acc				;
MoPar_51 :					;    END (* IF *) ;
	rjmp	MoPar_8				;   END (* CASE VARIANT *) ;
MoPar_5	:					;
	cpi	acc,Key6Code			;  | Key6Code  :
	brne	MoPar_6				;   BEGIN
	ldz	OptionSrv<<1			;    OptionSrv [ Option ] ( ) ;
	lds	acc,Option			;
	lsl	acc				;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;
	push	r0				;
	adiw	zl,1				;
	lpm					;
	mov	zh,r0				;
	pop	zl				;
	icall					;
	ret					;    RETURN ;
						;   END (* CASE VARIANT *) ;
MoPar_6	:					;
	cpi	acc,Key7Code			;  | Key7Code  :
	brne	MoPar_7				;   BEGIN
	ret					;    RETURN ;
						;   END (* CASE VARIANT *) ;
MoPar_7	:					;
MoPar_8	:					;  END (* CASE *) ;
	rjmp	MoPar_0				; END (* LOOP *) ;
	ret					;END (* ModiParamSrv *) ;
;-----------------------------------------------------------------------------
DefaultParamService :				;PROCEDURE DefaultParamService ( ) ;
;********************				;BEGIN (* DefaultParamService *)
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	ldz	ResetParamQn<<1			; WriteTextLCDFlash ( ResetParamQn ) ;
	rcall	WriteTextLCDFlash		;
DefaPar_3 :					; LOOP
DefaPar_0 :					;  REPEAT
	rcall	KeyPressed			;  UNTIL KeyPressed ( ) ;
	brcc	DefaPar_0			;
	rcall	ReadKey				;  CASE ReadKey ( ) OF
	cpi	acc,Key6Code			;  | Key6Code  :
	brne	DefaPar_1			;   BEGIN
	rcall	DefaultParam			;    DefaultParam ( ) ;
	ret					;    RETURN ;
DefaPar_1 :					;   END (* CASE VARIANT *) ;
	cpi	acc,Key7Code			;  | Key7Code  :
	brne	DefaPar_2			;   BEGIN
	ret					;    RETURN ;
DefaPar_2 :					;   END (* CASE VARIANT *) ;
						;  END (* CASE *) ;
	rjmp	DefaPar_3			; END (* LOOP *) ;
						;END (* DefaultParamService *) ;
;-----------------------------------------------------------------------------
RunCommand :					;PROCEDURE RunCommand ( Code[param] : BYTE ) ;
;***********					;BEGIN (* RunCommand *)
						; CASE Code OF
	cpi	param,Key0Code			; | Key0Code :
	brne	RunCo_0				;  BEGIN
	rcall	DisplayParam			;   DisplayParam ( ) ;
	ret					;   RETURN ;
						;  END (* CASE VARIANT *) ;
RunCo_0	:					;
	cpi	param,Key1Code			; | Key1Code :
	brne	RunCo_1				;  BEGIN
	rcall	DefaultParamService		;   DefaultParamService ( ) ;
	ret					;   RETURN ;
						;  END (* CASE VARIANT *) ;
RunCo_1	:					;
	cpi	param,Key2Code			; | Key2Code :
	brne	RunCo_2				;  BEGIN
	ret					;   RETURN ;
						;  END (* CASE VARIANT *) ;
RunCo_2	:					;
	cpi	param,Key3Code			; | Key3Code :
	brne	RunCo_3				;  BEGIN
	rcall	ModiParamSrv			;   ModiParamSrv ( ) ;
	ret					;   RETURN ;
						;  END (* CASE VARIANT *) ;
RunCo_3	:					;
	cpi	param,Key4Code			; | Key4Code :
	brne	RunCo_4				;  BEGIN

	ret					;   RETURN ;
						;  END (* CASE VARIANT *) ;
RunCo_4	:					;
	cpi	param,Key5Code			; | Key5Code :
	brne	RunCo_5				;  BEGIN

	ret					;   RETURN ;
						;  END (* CASE VARIANT *) ;
RunCo_5	:					;
	cpi	param,Key6Code			; | Key6Code :
	brne	RunCo_6				;  BEGIN

	ret					;   RETURN ;
						;  END (* CASE VARIANT *) ;
RunCo_6	:					;
	cpi	param,Key7Code			; | Key7Code :
	brne	RunCo_7				;  BEGIN

	ret					;   RETURN ;
						;  END (* CASE VARIANT *) ;
RunCo_7	:					; ELSE
						; END (* CASE *) ;
	ret					;END (* RunCommand *) ;
;-----------------------------------------------------------------------------
ResetProcessor	:				;
	cli					;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
Main	:					;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	rcall	LongDelay			; LongDelay ( ) ;
	rcall	InitialiseLCD			; InitialiseLCD ( ) ;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	sei					; EI ( ) ;
	rcall	LoadParam			; IF NOT LoadParam ( ) THEN
	brcs	Main_1				; BEGIN
	ldz	CRCFailTxt<<1			;  DisplayMessage ( CRCFailTxt ) ;
	rcall	DisplayMessage			;
	ldz	DefaParamTxt<<1			;  DisplayMessage ( DefaParamTxt ) ;
	rcall	DisplayMessage			;
	rcall	DefaultParam			;  DefaultParam ( ) ;
	ldz	SaveParamTxt<<1			;  DisplayMessage ( SaveParamTxt ) ;
	rcall	DisplayMessage			;
Main_1	:					; END (* IF *) ;
	ldz	HelloText<<1			; DisplayMessage ( HelloText ) ;
	rcall	DisplayMessage			;
	rcall	ClrScrLCD			; ClrScrLCD ( ) ;
	rcall	Delay				; Delay ( ) ;
	rcall	Delay				; Delay ( ) ;
Main_0	:					; LOOP
	lds	acc,DispMenuReq			;  IF DispMenuReq THEN
	cpi	acc,0				;
	breq	Main_3				;  BEGIN
	rcall	ClrScrLCD			;   ClrScrLCD ( ) ;
	ldz	SelectOptTxt<<1			;   WriteTextLCDFlash ( SelectOptTxt ) ;
	rcall	WriteTextLCDFlash		;
	ldi	acc,0				;   DispMenuReq := FALSE ;
	sts	DispMenuReq,acc			;
Main_3	:					;  END (* IF *) ;
	rcall	KeyPressed			;  IF KeyPressed ( ) THEN
	brcc	Main_2				;  BEGIN
	rcall	ReadKey				;   RunCommand ( ReadKey ( ) ) ;
	mov	param,acc			;
	rcall	RunCommand			;
	ldi	acc,1				;   DispMenuReq := TRUE ;
	sts	DispMenuReq,acc			;
Main_2	:					;  END (* IF *) ;

	rjmp	Main_0				; END (* LOOP *) ;
						;END Main ;
;-----------------------------------------------------------------------------
;			      CONST
;				OptionTxt : ARRAY [ 0 .. OptionLimit ] OF ADDRESS =
OptionTxt:	.dw	SelOption1Text		;     { SelOption1Text ,
		.dw	SelOption2Text		;	SelOption2Text ,
		.dw	SelOption3Text		;	SelOption3Text ,
		.dw	SelOption4Text		;	SelOption4Text } ;
;			      CONST
;				OptionSrv : ARRAY [ 0 .. OptionLimit ] OF PROCEDURE ( ) =
OptionSrv:	.dw	Option1Service		;     { Option1Service ,
		.dw	Option2Service		;       Option2Service ,
		.dw	Option3Service		;       Option3Service ,
		.dw	Option4Service		;       Option4Service } ;
;			      CONST
;				ValidParam3Table : ARRAY [ 0 .. 7 ] OF ADDRESS =
ValidPar3Table:	.dw	AVRTxt			;     { AVRTxt  ,
		.dw	AVR1Txt			;       AVR1Txt ,
		.dw	AVR2Txt			;       AVR2Txt ,
		.dw	AVR3Txt			;       AVR3Txt ,
		.dw	AVR4Txt			;       AVR4Txt ,
		.dw	AVR5Txt			;       AVR5Txt ,
		.dw	AVR6Txt			;       AVR6Txt ,
		.dw	AVR7Txt			;       AVR7Txt } ;
AVRTxt:		.db	"* AVR *",0
AVR1Txt:	.db	"*MEGA *",0
AVR2Txt:	.db	"*TINY *",0
AVR3Txt:	.db	"- cos -",0
AVR4Txt:	.db	"- inne-",0
AVR5Txt:	.db	"- lub -",0
AVR6Txt:	.db	"-takie-",0
AVR7Txt:	.db	"-same -",0
Opt2Prompt:	.db	"Nowy param2 =",0
Opt4Prompt:	.db	"Nowy param4 =",0
HelloText :	.db	"Obsluga EEPROM",0,0
CRCFailTxt :	.db	"Blad sumy CRC",0
DefaParamTxt :	.db	"Parametry zastepcze",0
SaveParamTxt :	.db	"Zapis parametrow",0,0
SelectOptTxt:	.db	"Wybierz opcje",0
ResetParamQn:	.db	"Czy utworzyc parametry zastepcze?",0
Param1PromptTxt:.db	"Param1 = ",0
Param2PromptTxt:.db	"Param2 = ",0
Param3PromptTxt:.db	"Param3 = ",0
Param4PromptTxt:.db	"Param4 = ",0
SelOption1Text:	.db	"Modyfikacja pola Param1",0
SelOption2Text:	.db	"Modyfikacja pola Param2",0
SelOption3Text:	.db	"Modyfikacja pola Param3",0
SelOption4Text:	.db	"Modyfikacja pola Param4",0
;-----------------------------------------------------------------------------
.exit
