;
;*******************************************************
;*******************************************************
;**                                                   **
;** Program prezentacji obslugi USART w przerwaniach  **
;**                                                   **
;**         Andrzej Pawluczuk                         **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                                                   **
;**             Bialystok, 2006                       **
;**                                                   **
;*******************************************************
;*******************************************************
;
	.nolist
;*******************************************************
.include "m8515def.inc"
;*******************************************************
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.def acc		= r16
.def acc2		= r17
.def param		= r18
;-----------------------------------------------------------------------------
.equ		Stand_TIMSK	=	0b00000010
;	Timer/counter Interrupt Mask	  ||| | |
;					  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;					  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;					  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;					  |+-------- OCE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;					  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ		Stand_TCCR0	=	0b00000011
;	Timer/Counter0 Control Register	       |||
;					       ||+-- CS00
;					       |+--- CS01
;					       +---- CS02
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.equ		Stand_PADir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PAData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PBDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : RXD LED
;					  +--------- output : TXD LED
.equ		Stand_PBData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PCDir	=	0b11111111
;					  ||||||||
;					  |||||||+-- output : 
;					  ||||||+--- output : 
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PCData	=	0b11111111
;-----------------------------------------------------------------------------
.equ		Stand_PDDir	=	0b11111110
;					  ||||||||
;					  |||||||+-- input :	[RXD]
;					  ||||||+--- output :	[TXD]
;					  |||||+---- output : 
;					  ||||+----- output : 
;					  |||+------ output : 
;					  ||+------- output : 
;					  |+-------- output : 
;					  +--------- output : 
.equ		Stand_PDData	=	0b11111111
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;					;CONST
.equ	Cr		=	0x0D	; Cr = 0DH ;
.equ	Lf		=	0x0A	; Lf = 0AH ;
.equ	TXDBuffSize	=	20	; TXDBuffSize = 20 ;	(* wielkosc bufora nadajnika -> usartlib.inc *)
.equ	RXDBuffSize	=	16	; RXDBuffSize = 16 ;	(* wielkosc bufora odbiornika -> usartlib.inc *)
;					;
;					;VAR
Character:	.byte	1		; Character : CHAR ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;USART Receive Complete Interrupt Vector Address
	rjmp	USARTRXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;USART Data Register Empty Interrupt Vector Address
	rjmp	USARTDataRegEmptInterrupt	;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;USART Transmit Complete Interrupt Vector Address
	rjmp	USARTTXDInterrupt		;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.equ	USART_Speed	=	47	;
;	USART BAUD Rate Generator fosc=7.3728 MHz , 9600 bps
.equ	USART_LED_Trace	=	1	; wlacza sygnalizacje transmisji poprzez diody LED
.equ	USARTTracePort	=	PORTB	;
.equ	RXDLED		=	6	;
.equ	TXDLED		=	7	;
.equ	LEDOnState	=	0	;
.include "usartlib.inc"
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN
	ldi	acc,Stand_PADir			; [DDRA] := Stand_PADir ;
	out	DDRA,Acc			;
	ldi	acc,Stand_PAData		; [PORTA] := Stand_PAData ;
	out	PORTA,acc			;
	ldi	acc,Stand_PBDir			; [DDRB] := Stand_PBDir ;
	out	DDRB,Acc			;
	ldi	acc,Stand_PBData		; [PORTB] := Stand_PBData ;
	out	PORTB,acc			;
	ldi	acc,Stand_PCDir			; [DDRC] := Stand_PCDir ;
	out	DDRC,Acc			;
	ldi	acc,Stand_PCData		; [PORTC] := Stand_PCData ;
	out	PORTC,acc			;
	ldi	acc,Stand_PDDir			; [DDRD] := Stand_PDDir ;
	out	DDRD,Acc			;
	ldi	acc,Stand_PDData		; [PORTD] := Stand_PDData ;
	out	PORTD,acc			;
	ldz	USART_Speed			; USARTHardwareInit ( USART_Speed ) ;
	rcall	USARTHardwareInit		;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN
	rcall	USARTSoftwareInit		; USARTSoftwareInit ( ) ;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				;BEGIN
	pushf					;
	rcall	USARTTimeService		; USARTTimeService ( ) ;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
SendNewLine :					;PROCEDURE SendNewLine ( ) ;
;************					;BEGIN
	ldi	param,Cr			; SendSerial ( Cr ) ;
	rcall	SendSerial			;
	ldi	param,Lf			; SendSerial ( Lf ) ;
	rcall	SendSerial			;
	ret					;END SendNewLine ;
;-----------------------------------------------------------------------------
Anal	:					;PROCEDURE Anal ( ) ;
;********					; VAR
;						;  Loop [acc2] : BYTE ;
;						;  Ch[acc] : BYTE ;
;						;BEGIN
	lds	param,Character			; IF Character >= ' ' THEN
	cpi	param,' '			;
	brcs	Anl_0				; BEGIN
	rcall	SendSerial			;  SendSerial ( Character ) ;
Anl_0	:					; END (* IF *) ;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ldz	AnalText1<<1			; SendSerialFlashTxt ( AnalText1 ) ;
	rcall	SendSerialFlashTxt		;
	lds	acc,Character			; Ch := Character ;
	ldi	acc2,0				; FOR Loop := 0 TO 7 DO
Anl_1	:					; BEGIN
	rol	acc				;  IF Ch.7 = 1 THEN
	brcc	Anl_2				;  BEGIN
	ldi	param,'1'			;   SendSerial ( '1' ) ;
	rjmp	Anl_3				;  END (* IF ... *)
Anl_2	:					;  ELSE
						;  BEGIN
	ldi	param,'0'			;   SendSerial ( '0' ) ;
Anl_3	:					;  END (* IF ... ELSE *) ;
	push	acc				;  Ch := Ch * 2 ;
	push	acc2				;
	rcall	SendSerial			;
	pop	acc2				;
	pop	acc				;
	cpi	acc2,7				;
	breq	Anl_4				;
	inc	acc2				;
	rjmp	Anl_1				;
Anl_4	:					; END (* FOR *) ;
	ldz	AnalText2<<1			; SendSerialFlashTxt ( AnalText2 ) ;
	rcall	SendSerialFlashTxt		;
	ldz	HexConvTable<<1			; SendSerial ( HexConvTable [ Character DIV 16 ] ) ;
	lds	acc,Character			;
	swap	acc				;
	andi	acc,0x0F			;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;
	mov	param,r0			;
	rcall	SendSerial			;
	ldz	HexConvTable<<1			; SendSerial ( HexConvTable [ Character MOD 16 ] ) ;
	lds	acc,Character			;
	andi	acc,0x0F			;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	lpm					;
	mov	param,r0			;
	rcall	SendSerial			;
	rcall	SendNewLine			; SendNewLine ( ) ;
	ret					;END (* Anal *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
	ldz	HelloText<<1			; SendSerialFlashTxt ( HelloText ) ;
	rcall	SendSerialFlashTxt		;
Main_0	:					; LOOP
	ldz	PromptText<<1			;  SendSerialFlashTxt ( PromptText ) ;
	rcall	SendSerialFlashTxt		;
Main_1	:					;  REPEAT
	rcall	SerialDataPresent		;  UNTIL SerialDataPresent ( ) ;
	brcc	Main_1				;
	rcall	GetSerial			;  Character := GetSerial ( ) ;
	sts	Character,acc			;
	rcall	Anal				;  Anal ( ) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *).
;-----------------------------------------------------------------------------
HexConvTable:	.db	"0123456789ABCDEF"
HelloText:	.db	Cr,Lf,Cr,Lf,"Hello !",Cr,Lf,Cr,Lf,0
PromptText:	.db	"Nacisnij jakis znak: ",0
AnalText1:	.db	"Postac binarna znaku : " , 0
AnalText2:	.db	" postac szesnastkowa znaku : " , 0
.exit
